#!/usr/local/bin/perl

# ------------------------------------------------------------------
# dbamonTask.pl
#
#  This program is invoked by /opt/dbamon/bin/dbamon. It performs
#  DBAmon processing for 1 task. 
#
# ------------------------------------------------------------------

#---- Global settings
push(@INC,"/opt/dbamon/lib");
push(@INC,"/opt/dbamon/adm");
$| = 1;

use POSIX 'WNOHANG';

#---- Signal handling
$SIG{'INT'} = 'intHandler';
sub intHandler
{
   local($signal) = @_;
   print "DBAmon killed by signal=$signal\n";
   exit(0);
}

$SIG{ALRM} = \&ALARMCHECK;
sub ALARMCHECK 
{
  # Kill remaining children
  local $SIG{HUP} = sub { print "DBAmon Internal - HUP_Signal\n"};
  kill('HUP', -$$);
  # Issue message and return
  die "ALRM_Signal_TIMEOUT" ;
}

#$SIG{CHLD} = \&REAPER;
sub REAPER
{
  while( ( my $kid = waitpid( -1, WNOHANG ) ) > 0 )
  {
    print "DBAmon Internal - CHLD_Signal kid=$kid\n";
    $sig_chld_flag = 1;
  }
  print "DBAmon Internal - CHLD_Signal\n" if $sig_chld_flag != 1;
  $SIG{CHLD} = \&REAPER;
}

#---- Include Subroutines
require "subs.pl" ;
require "subsDB.pl" ;
require "varsInit.pl" ;

require "debug.pl" ;
require "logit.pl" ;
require "heartbeat.pl" ;
require "readConfig.pl" ;
require "readDBCFile.pl" ;
require "readDebug.pl" ;
require "readCMDBContacts.pl" ;
require "readDist.pl" ;
require "readSWVer.pl" ;
require "readTaskHistory.pl" ;
require "readGenesis.pl" ;
require "readEventSnooze.pl" ;
require "error_parm_null.pl" ;
require "error_parm.pl" ;
require "error_msg.pl" ;
require "log_msg.pl" ;
require "check_sw.pl" ;
require "begin_stuff.pl" ;
require "run_monitor.pl" ;
require "check_inhib.pl" ;
require "remsh.pl" ;
require "function_cleanup.pl" ;
require "logDetails.pl" ;
require "log_file.pl" ;
require "log_notify.pl" ;
require "log_snooze.pl" ;
require "history_dummy.pl" ;
require "histCreate.pl" ;
require "metricsInsert.pl" ;
require "metricsCurrent.pl" ;
require "end_stuff.pl" ;
require "summarize.pl" ;
require "summarizePulldown.pl" ;
require "sumprint.pl" ;
require "sumTagSelect.pl" ;
require "sumCMDBSelect.pl" ;
require "sumtot.pl" ;
require "event.pl" ;
require "eventGraphPerf.pl" ;
require "eventHistory.pl" ;
require "eventLog.pl" ;
require "eventOCheck.pl" ;
require "eventNotify.pl" ;
require "eventNotifyCMDB.pl" ;
require "eventDB.pl" ;
require "eventDupCheckDB.pl" ;
require "baICheck.pl" ;
require "uptimeLog.pl" ;
require "uptimeCalc.pl" ;
require "noteCount.pl" ;
require "dbmsPatchCreate.pl" ;
require "verifyDirs.pl" ;
require "verifyTables.pl" ;
require "instanceDetails.pl" ;
require "sumupdate.pl" ;       #---- 2010/12/21 BB
require "snapshotsInsert.pl";  #---- 2013/04/23  
require "snapshotsKeyGen.pl"; 
require "instanceBookmarkInsert.pl";    #---- 2014/01/08
require "instanceBookmarkCheck.pl";    #---- 2014/01/08

require "DownloadSW.pl";
require "exportVars.pl";
require "DownloadSWWin.pl";
require "downloadFlagOff.pl";
require "sqlReqFile.pl";

require "sqlUp.pl";
require "sqlTLog.pl";
require "sqlBkup.pl";
require "sqlBkupTLog.pl";
require "sqlAlerts.pl";
require "sqlFG.pl";      #---- New 2011
require "sqlPerf.pl";    #---- New 2011
require "sqlJobs.pl";    #---- New 2011
require "sqlSaveConfig.pl";    #---- New 2011
require "sqlSvcAcctSelect.pl"; #---- New 2013/05/08
require "sqlTrends.pl";  #---- New 2013/07/07

require "osWinUp.pl";    #---- New 2014
require "osWinReqFile.pl";
require "osWinPerf.pl";    #---- New 2011

require "orantUp.pl";
require "orantTSFull.pl";
require "orantArclog.pl";
require "orantDatafiles.pl";
require "orantBkup.pl";
require "orantPerf.pl";
require "orantPerf2.pl";

require "infUp.pl";
require "infLongTX.pl";
require "infChkDup.pl";
require "infSWVer.pl";
require "infChunksDown.pl";
require "infSaveCfg.pl";
require "infLocks.pl";
require "infNFile.pl";
require "infCkpt.pl";
require "infShmem.pl";
require "infExtents.pl";
require "infDBS.pl";
require "infCleaners.pl";
require "infLlogs.pl";
require "infArc.pl";
require "infHDR.pl";
require "infMLog.pl";
require "infPerf.pl";

require "oraUp.pl";
require "oraSec.pl";
require "oraSaveCfg.pl";
require "oraLsnr.pl";
require "oraDatafiles.pl";
require "oraBkup.pl";
require "oraBkupArc.pl";
require "oraTSFull.pl";
require "oraDGFull.pl";
require "oraAlerts.pl";
require "oraExtents.pl";
require "oraArclog.pl";
require "oraSGA.pl";
require "oraInSync.pl";
require "oraPerf.pl";
require "oraAuditSes.pl";

require "oraapUp.pl";

require "sapUp.pl";

#---- Include DBAmon Main Section
require "dbamonMain.pl" ;

#---- Terminate
exit 0;
