#!/usr/local/bin/perl
#--------------------------------------------------------------------------
#
# dbamon_oswin.pl
#
#--------------------------------------------------------------------------
# Changes:
# 1.00 16Jun14 bb - Initial
# 1.01 13Sep15 bb - For drive logic, exclude any drive whose label contains /pag/i. 
#--------------------------------------------------------------------------
$version = "1.01";

$| = 1;
$P = "dbamon_oswin.pl";

#--------------------------------------------------------------------------
#
# Global Variables
#
#--------------------------------------------------------------------------

#---- Behavior flags
$usewin32registry = 0; 
$use_mssqlregread = 1; 
$use_wmiosver     = 1; 
$use_rr_wow6432   = 0;   #---- This can be changed later

#---- Global Settings
$event_limit         = 1000000;
$typeperf_sample_dur = 1;
$debug               = 0;
$use_wmio            = 1; 
$repl_dist           = 0; 
$style_normal = "style='font-family:Arial;font-size:10pt;'";

$deadlock_flag       = 0; 
$deadlock_lastts     = '9999'; 

use Time::Local;
use DBI;

#---- for WMI calls
use constant wbemFlagReturnImmediately => 0x10;
use constant wbemFlagForwardOnly => 0x20;

#---- Get rid of compile warnings
$bt_desc = $tsts = $jobsupdated = $fg_fileid = $fg_growth = $char_val = $size = $usr2 = $root_fs = $hoursup = $endc = $pgm2 = $fg_file_in_group = $index_val = ""; 
$bt_dbname = $bt_type = $bt_timestamp = $bt_size  = ""; 

#------------------------------------------------------------------------------
#
# Subroutines
#
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
sub commify 
#------------------------------------------------------------------------------
{
  # commify a number. Perl Cookbook, 2.17, p. 64
  my $text = reverse $_[0];
  $text =~ s/(\d\d\d)(?=\d)(?!\d*\.)/$1,/g;
  return scalar reverse $text;
}

#------------------------------------------------------------------------------
sub timeToNum
#------------------------------------------------------------------------------
{
  $zz_text = $_[0];
  $zz_text =~ s/-/\//g;
  $zz_text =~ s/:/\//g;
  ##print "zz_text=$zz_text\n";
  @words = split(/\//,$zz_text);

  if ($#words != 5)
  {
    &error("Input Target_End_Time ($zz_text) should be in the format: YYYY/MM/DD-HH:MM:SS","DOC116");
  }

  $zz_yyyy = $words[0];
  $zz_mon  = $words[1];
  $zz_dd   = $words[2];
  $zz_hh   = $words[3];
  $zz_mm   = $words[4];
  $zz_ss   = $words[5];
  ##print "zz_yyyy=$zz_yyyy zz_mon=$zz_mon zz_dd=$zz_dd zz_hh=$zz_hh zz_mm=$zz_mm zz_ss=$zz_ss\n";

  &error("Input Target_End_Time - YYYY must be 2002-2099","DOC116") if ($zz_yyyy < 2002 || $zz_yyyy > 2099);
  &error("Input Target_End_Time - MON  must be 1-12     ","DOC116") if ($zz_mon  <    1 || $zz_mon  >   12);
  &error("Input Target_End_Time - DD   must be 1-31     ","DOC116") if ($zz_dd   <    1 || $zz_dd   >   31);
  &error("Input Target_End_Time - HH   must be 0-23     ","DOC116") if ($zz_hh   <    0 || $zz_hh   >   23);
  &error("Input Target_End_Time - MM   must be 0-59     ","DOC116") if ($zz_mm   <    0 || $zz_mm   >   59);
  &error("Input Target_End_Time - SS   must be 0-59     ","DOC116") if ($zz_ss   <    0 || $zz_ss   >   59);

  @zz_date=($zz_ss,$zz_mm,$zz_hh,$zz_dd,--$zz_mon,$zz_yyyy);
  $zz_result = &timelocal(@zz_date);

  return $zz_result;
}

# ----------------------------------------------------------------------
sub getDBAttrs2000
# ----------------------------------------------------------------------
{
  my $dbname = $_[0];
  &msg("getDBAttrs2000 - Starting - dbname=$dbname"); 

  #---- dbamon_oswin.pl DB_ATTRS: model DBState=ONLINE,CompatibilityLevel=100,CreateDate=2003/04/08-09:13:36,DBID=3,RecoveryModel=SIMPLE

  $sth = $dbh->prepare("exec sp_helpdb \@dbname='$dbname'") || &dbError("Prepare");
  $sth->execute || &dbErrorOK("sp_helpdb db=$dbname");

  do
  {
    while(@Data = $sth->fetchrow_array)
    {
      if ($Data[0] eq $dbname) 
      { 
        my $zdata = $Data[5]; 
        $zdata =~ s/\s+//g;
        &dataReturn("DB_ATTRS: $dbname $zdata"); 
        $dbattrs{$dbname} = $zdata; 
      }
    }
  } while($sth->{odbc_more_results});
}

#------------------------------------------------------------------------------
sub getDBAttrs
#------------------------------------------------------------------------------
{
  #---- 1.26 START - Gather DB attributes (2005+ only)
  # 2012/06/26 BB - if ($ver >= 9)
  if ($vershort >= 90)
  {
    &msg("DEBUG: This is an MSSQL 2005+ DB - Checking for DB attributes") if $debug;
    $sql = "
    select
      source_database_id, 
      is_read_only ,
      compatibility_level,
      recovery_model_desc,
      database_id,
      tempdb.dbo.dateconvert_19(create_date),
      state_desc
    from sys.databases where name = '$dbname';
    ";
    &msg("DEBUG: Running sql=$sql") if $debug;

    $sth = $dbh->prepare("$sql") || &error("Prepare");
    $sth->execute || &error("Execute_Select");
    $db_attrs = ""; 
    while (@Data = $sth->fetchrow_array)
    { 
      $source_database_id = $Data[0];
      $is_read_only       = $Data[1];
      $comp_level         = $Data[2];
      $rec_model          = $Data[3];
      $database_id        = $Data[4];
      $cre_date           = $Data[5];
      $db_state           = $Data[6];
      &msg("DEBUG: source_database_id=$source_database_id is_read_only=$is_read_only") if $debug;
    }

    $db_attrs .= "DBState=$db_state,CompatibilityLevel=$comp_level,CreateDate=$cre_date,DBID=$database_id,RecoveryModel=$rec_model";

    if ($source_database_id > 0) { $db_attrs .= ",Snapshot"; }
    if ($is_read_only == 1)      { $db_attrs .= ",Readonly"; }

    $dbname_out = $dbname;
    $dbname_out =~ s/\s/[BLANK]/g;
    $dbname_out =~ s/\-/[HYPHEN]/g;

    if ($db_attrs)
    {
      ##chop $db_attrs;    #---- Remove trailing comma
      &dataReturn("DB_ATTRS: $dbname_out $db_attrs"); 
      $dbattrs{$dbname} = $db_attrs; 
    }
  }
  else
  {
    &msg("DEBUG: This is NOT a MSSQL 2005+ Instance - Calling getDBAttrs2000");
    &getDBAttrs2000($dbname); 
  }
  #---- 1.26 END

  #---- 2.13 START
  $sql = "SELECT 
  'AutoShrink', 
  DATABASEPROPERTYEX ( '$dbname' , 'isautoshrink' ) 
  UNION ALL
  select 
  'AutoClose',  
  DATABASEPROPERTYEX ( '$dbname' , 'isautoclose' ) 
  order by 1
  "; 
  $sth = $dbh->prepare($sql) || &dbError("Prepare_L3546");
  $sth->execute || &dbErrorOK("execute helpdb db=$dbname");

  $badparms = ""; 
  while (@Data = $sth->fetchrow_array)
  {
    $colname = $Data[0]; 
    $value   = $Data[1]; 
    if ($value == 1) 
    { 
      $z = uc $colname;
      $badparms .= "$z=$value,"; 
    }
  }
  &msg("BADPARMS Results - badparms=($badparms)") if ! $WWW;
  if ($badparms)
  {
    $badparms =~ s/\,$//;  #---- Remove trailing comma
    $dbname_out = $dbname;
    $dbname_out =~ s/\s/[BLANK]/g;
    $dbname_out =~ s/\-/[HYPHEN]/g;
    &dataReturn("BADPARMS: $dbname_out $badparms"); 
  }
  #---- 2.13 END
  
  #---- 2.43 START - Look for good parms that are off but should be on
  $sql = "SELECT 
  'AutoCreateStatistics', 
  DATABASEPROPERTYEX ( '$dbname' , 'isautocreatestatistics' ) 
  UNION ALL
  select 
  'AutoUpdateStatistics',  
  DATABASEPROPERTYEX ( '$dbname' , 'isautoupdatestatistics' ) 
  order by 1
  "; 
  $sth = $dbh->prepare($sql) || &dbError("Prepare_L3546");
  $sth->execute || &dbErrorOK("execute helpdb db=$dbname");

  my $goodparms = ""; 
  while (@Data = $sth->fetchrow_array)
  {
    $colname = $Data[0]; 
    $value   = $Data[1]; 
    if ($value == 0) 
    { 
      $z = uc $colname;
      $goodparms .= "$z=$value,"; 
    }
  }
  &msg("GOODPARMS Results - goodparms=($goodparms)") if ! $WWW;
  if ($goodparms)
  {
    $goodparms =~ s/\,$//;  #---- Remove trailing comma
    $dbname_out = $dbname;
    $dbname_out =~ s/\s/[BLANK]/g;
    $dbname_out =~ s/\-/[HYPHEN]/g;
    &dataReturn("GOODPARMS_Off: $dbname_out $goodparms"); 
  }
  #---- 2.43 END
}

#------------------------------------------------------------------------------
sub get_date
#------------------------------------------------------------------------------
{
  # Return date in format supplied as argument. Valid formats are:
  #   Y/M/D-H:M:S
  #   YMD_HMS
  #   YMD
  $format = $_[0];

  @sysdate = localtime;
  for ($i = 0; $i <= 5; $i++)
  {
    if ($sysdate[$i] < 10)
    {
      $sysdate[$i] = 0 . $sysdate[$i];
    }
  }

  if ($format eq "Y/M/D-H:M:S")
  {
    $sysdate = $sysdate[5]+1900 . "/" . ++$sysdate[4] . "/" . $sysdate[3] . "-" . $sysdate[2] . ":" . $sysdate[1] . ":" . $sysdate[0];
  }
  elsif ($format eq "YMD_HMS")
  {
    $sysdate = $sysdate[5]+1900 . ++$sysdate[4] . $sysdate[3] . "_" . $sysdate[2] . $sysdate[1] . $sysdate[0];
  }
  elsif ($format eq "YMD")
  {
    $sysdate = $sysdate[5]+1900 . ++$sysdate[4] . $sysdate[3];
  }
  else
  {
    &error("Invalid format passed to get_date. Must be Y/M/D-H:M:S or YMD_HMS");
  }
  return $sysdate;
}

#------------------------------------------------------------------------------
sub msg
#------------------------------------------------------------------------------
{
  $msgmsg = $_[0];
  $date = &get_date("Y/M/D-H:M:S");
  print "$date ${P} $version | $msgmsg\n";
  #---- Also log to a file
  print PLOG "$date ${P} $version | $msgmsg\n";
}

#------------------------------------------------------------------------------
sub dbError 
#------------------------------------------------------------------------------
{
   local($errorMsg,@error_fields) = @_;
   print "======== D B   E r r o r   E n c o u n t e r e d =========\n";
   print "Error: DB Error Encountered While: $errorMsg\n";
   print "Error: DB Error String: $DBI::errstr\n";
   print "LastSQL:  $sql\n" if $sql ne "";
   print "==========================================================\n";
   $dberror = 1;
   @errors = split ("Database",$DBI::errstr);
   $chk_str = lc($errors[1]);
   $db      = $dbname;
   if ($chk_str =~ " \'$db\' cannot be opened because it is offline")
     {
       $offline = 1;
     }
   &msg("END_OF_PROBE");
   exit 1;
}

#------------------------------------------------------------------------------
sub dbErrorOK 
#------------------------------------------------------------------------------
{
   local($errorMsg,@error_fields) = @_;
   print "======== D B   E r r o r   E n c o u n t e r e d =========\n";
   print "Error: DB Error Encountered While: $errorMsg\n";
   print "Error: DB Error String: $DBI::errstr\n";
   print "LastSQL:  $sql\n" if $sql ne "";
   print "==========================================================\n";
   $dberror = 1;
   @errors = split ("Database",$DBI::errstr);
   $chk_str = lc($errors[1]);
   $db      = $dbname;
   if ($chk_str =~ " \'$db\' cannot be opened because it is offline")
     {
       $offline = 1;
     }
}

# ----------------------------------------------------------------------
sub error
# ----------------------------------------------------------------------
{
  $msg = $_[0];
  print "$P | Fatal Error: $msg\n";
  print "***********************************************************\n";
  print "Error: $msg\n";
  print "***********************************************************\n";
  &msg("<font color=red><b>ERROR: $msg</b></font>");
  print "$P | Exiting\n";
  &msg("END_OF_PROBE");
  close(PLOG); 
  exit 1;
}

# ----------------------------------------------------------------------
sub bkbl
# ----------------------------------------------------------------------
{
  $dbname = $arg1;
  $bktype = $arg2;

  $dbname =~ s/\[HYPHEN\]/\-/g;
  $dbname =~ s/\[BLANK\]//g;
  
  if ($dbname eq "ALL")
  {
    if (-e "$SQLPATH\\dba_bkup_log\\ALL.txt")
    {
      $logpath = "$SQLPATH\\dba_bkup_log\\ALL.txt";
    }
    else
    {
      &msg("Looking for ALL.log in alternate location");
      $logpath = "\\temp\\ALL.txt";
    }
  }
  else
  {
    $logpath = "$SQLPATH\\dba_bkup_log\\${dbname}_${bktype}_backlog.txt";
  }

  &msg("bkbl1() - dbname=$dbname bktype=$bktype logpath=$logpath");

  if (! -e $logpath)
  {
    &error("Log file $logpath does not exist");
  }

  print "<table bgcolor=skyblue class=c1>
  <tr><td><pre>
<b>Start of mssql_backup Log: $logpath</b><hr>";

# - added by Scott.  I added the \" around the $logpath variable to accept spaces
# - in the file name and/or path.
  system("type \"$logpath\" 2>&1");

  print "<hr>
<b>End of mssql_backup Log: $logpath</b></table>\n"; 
}

# ----------------------------------------------------------------------
sub hashDescTS 
# ----------------------------------------------------------------------
{
  $b <=> $a
}

# ----------------------------------------------------------------------
sub getNetInfo
# ----------------------------------------------------------------------
{
  &msg("getNetInfo - Starting - Querying net properties"); 

  #---- IP and VLAN
  open(NET, 'ipconfig|') || print "Error: Running net start - $!\n";
  while(<NET>)
  {
    ##print;
    @words = split;

    if (/IPv4 Address/) 
    {
      $net_ip = $words[$#words]; 
    }
    if (/IP Address/) 
    {
      $net_ip = $words[$#words]; 
    }
  }

  #---- Extract VLAN 
  @net_vlan_ary = split (/\./,$net_ip); 
  $net_vlan = $net_vlan_ary[0] . "." . $net_vlan_ary[1] . "." . $net_vlan_ary[2]; 

  &dataReturn("NET_IP:      $net_ip");
  &dataReturn("NET_VLAN:    $net_vlan");
  &dataReturn("NET_COMPANY: $company");

  &msg("getNetInfo - Done - net_ip=$net_ip net_vlan=$net_vlan"); 
}

# ----------------------------------------------------------------------
sub up
# ----------------------------------------------------------------------
{
  &msg("up() Starting - arg1=$arg1 arg2=$arg2"); 
  
  #---- Get Network stuff
  &getNetInfo;

  #---- See if this is a PHYSICAL or VIRTUAL server (presence of c:\program files\vmware folder)
  if ( -e "c:\\program files\\vmware") { $server_type = "VMWare"; }
  else                                 { $server_type = "Physical"; }
  &dataReturn("ServerType: $server_type");

  #---- OS Version Info from WMI 
  &WMIOSVer; 
  
  #---- OS Info from WMI (Last reboot) 
  &WMIOperatingSystem if $use_wmio; 

  #---- Services Info from WMI. Stored in a hash for other calls to use
  &WMIService; 

  #---- CPU Info from WMI
  &WMIProcessor; 

  #---- CPU Info from WMI
  &WMIComputerSystem; 

  #---- Gather network perf data
  &WMINetPerf; 
  
  #---- Gather top CPU consumers
  &WMITopProcessesCPU; 
  
  #---- Gather SCSI Controller info
  &WMISCSI; 
  
  #---- Count CPUs IF we didn't already
  &countCPUs if $numcpus eq "" || $numcpus == 0;

  #---- 3.10: Optionally add some drives to the list of drives that we will monitor
  if ($rq_mssql_addtl_drives ne "") 
  {
    @zchars = split(//, $rq_mssql_addtl_drives);
    foreach $this_drive (sort @zchars)
    {
      &msg("Adding drive this_drive=$this_drive to \@db_drivelist");
      push(@db_drivelist, $this_drive);
    }
  }

  #---- See if the MSSQLServer is running
  $rsh_service = "?";
  &msg("Running 'net start' to view running services - Looking for MSSQL running for this instance"); 
  open(NET, 'net start|') || print "Error: Running net start - $!\n";
  while(<NET>)
  {
    ##print;
    print if /sql/i;
    @words = split;

    if ($rsh_service eq "?")
    {
      if (/Remote Shell Daemon/)  { $rsh_service="RSD"; }
      if (/Remote Shell Service/) { $rsh_service="RSS"; }
      if (/CYGWIN sshd/)          { $rsh_service="CYGWIN"; }
      if (/openssh/i)             { $rsh_service="OpenSSH"; }
    }
  }
  close(NET);
  &msg("Read through running services - default_instance=$default_instance - servicestat=$servicestat"); 

  &dataReturn("HOSTNAME: $host");
  &dataReturn("RSHService: $rsh_service");

  #---- Get SysPerf data here... 
  &SysPerfBulk; 

  if (defined($ram) && !$WWW)
  {
     &dataReturn("RAM: $ram MB");
  }

  #---- Take 1st sp_monitor sample
  #---- 2011/02/10 BB - We don't need this any more since i now use typeperf
  ## &runSPMonitor;

  #---- Collect info about local drives
  &driveCheck; 

  &msg("Disk info:");
  foreach $drive (sort keys %disks)
  {
    &msg("  Disk: $drive - Size=$disk_size_gb{$drive} Free=$disk_free_gb{$drive} Used=$disk_used_gb{$drive}  PctF=$disk_pct_full{$drive}"); 
  }

  #---- 2011/04/08 - We only want to run SYSTEMINFO if we did not collect $mem_avail in SysPerf - it takes way too long
  #---- 2011/04/08 -   Note that i don't care about system_hardware_info - it isn't passed back to MASTER 
  if ($mem_avail eq "") 
  {
    #---- Systeminfo START - gather MEM usage info
    if ($osver >= 5.2 && -e "$ENV{'windir'}\\system32\\systeminfo.exe")
    {
      &msg("Running systeminfo to gather memory usage information"); 
      $system_hardware_info = ""; 
      open(SYSINFO, "systeminfo 2>&1 |") || &msg("Error: Running systeminfo - $!");
      while(<SYSINFO>)
      {
	##print;
	@words = split;
	if (/^Available Physical Memory:/)
	{
	  $mem_avail_raw = $words[$#words - 1] . $words[$#words];
	  $mem_avail = $words[$#words - 1] if $words[$#words] eq "MB";
	  $mem_avail =~ s/\,//g;
	} 
	if (/^System Boot Time:/) { $system_boot_time = $words[3] . $words[4] . $words[5]; }
	##if (/^System Manufacturer:/) { $system_hardware_info .= $words[2] . ":"; }
	if (/^System Model:/)     { $system_hardware_info .= "$words[2]_$words[3]_$words[4]_$words[5]";  }
	if (/^System Type:/)      { $system_hardware_info .= "_$words[2]_$words[3]";  }
      }
      close(SYSINFO); 
      
      &msg("DEBUG - mem_avail=$mem_avail mem_avail_raw=$mem_avail_raw ram=$ram mem_used=$mem_used mem_pct=$mem_pct"); 
      if ($mem_avail_raw ne "")
      {
	#TEMP#open(MEM, ">>c:\\temp\\MemAvail.txt") || &msg("Error: Cannot open output file: MemAvail.txt - $!");
	open(MEM, "MemAvail.txt") || &msg("Error: Cannot open output file: MemAvail.txt - $!");
	print MEM "$date $mem_avail_raw\n";
	close(MEM);
      }
    }
    #---- Systeminfo END - gather MEM usage info
  }

  #---- This could have come from SysPerf or from the above SYSTEMINFO
  #print "DEBUG28 - ram=$ram mem_avail=$mem_avail\n"; 
  if ($ram > 0 && $mem_avail ne "")
  {
    $mem_used = $ram - $mem_avail;
    $mem_pct  = sprintf("%.2f",($mem_used / $ram) * 100); 
  }
  print "DEBUG28B - mem_used=$mem_used mem_pct=$mem_pct\n"; 

  #---- Cluster status - START
  $cluster_name = "";
  open(CLUSTERV, "cluster /ver 2>&1 |"); 
  while(<CLUSTERV>)
  {
    ##print;
    #Cluster Name:     LSCASQLC
    @words = split;
    $cluster_name = $words[2] if (/^Cluster Name:/)
  }
  close(CLUSTERV); 
  &msg("cluster /ver Shows cluster_name=$cluster_name"); 
  if ($cluster_name ne "") 
  {
    &msg("Querying cluster status - $cluster_name"); 
    open(CLUSTER, "cluster /cluster:$cluster_name res |") || &error("Error: Running cluster");
    $cluster_status = "";
    while(<CLUSTER>)
    {
      ##print;
      @words = split;
      if (/^SQL Server  /)
      {
        $cluster_status = $words[$#words - 0];
        $cluster_node   = $words[$#words - 1];
        if ($cluster_status ne "Online")
        {
          $cluster_thisnodestatus   = "Offline"; 
        }
        else
        {
	  if ($cluster_node eq uc $host) { $cluster_thisnodestatus   = "Active"; }
	  else                           { $cluster_thisnodestatus   = "Passive"; }
    
        }
        ##&msg("  cs=$cluster_status cn=$cluster_node"); 
        ##print;
      }
    }
    close(CLUSTER); 

    &dataReturn("CLUSTERNAME: $cluster_name"); 
    &dataReturn("CLUSTERSTATUS: $cluster_status"); 
    &dataReturn("CLUSTERONNODE: $cluster_node"); 
    &dataReturn("CLUSTERTHISNODESTATUS: $cluster_thisnodestatus"); 
  }
  #---- Cluster status - END

  #---- Calc CPU BUSY - We did this earlier also
  #---- 2011/11/14 BB - Let's not do this any more - arithmetic exceptions on servers that have been up for a long time
  ##&runSPMonitor; 
  ##msg "$P CPU_BUSY_WITH_SQL: $cpu_busy\n"; 

  #---- Drives
  &msg("DEBUG - Rechecking disk drives"); 
  foreach (sort @db_drivelist)
  {
    $this_drive = $_;
    &msg("DEBUG - Found this_drive=$this_drive"); 
    if ($_ ne $last)
    {
      #---- Old
      #$this_drive_size   = t(" ",$disk_size_gb{"$this_drive"})) [2];
      #$this_drive_free   = (split(" ",$disks{"$this_drive"})) [3];
      #$this_drive_used   = (split(" ",$disks{"$this_drive"})) [4];
      #$this_drive_pctused = sprintf("%.2f",($this_drive_used / $this_drive_size) * 100) if $this_drive_size > 0;
      #---- New
      &msg("Checking drive: $this_drive");

      $this_drive_size   = commify($disk_size_mb{"$this_drive"});
      $this_drive_free   = commify($disk_free_mb{"$this_drive"});
      $this_drive_used   = commify($disk_used_mb{"$this_drive"});
      $this_drive_pctused = $disk_pct_full{"$this_drive"};

      if    ($this_drive_pctused >= 90) { $color = "pink"; }
      elsif ($this_drive_pctused >= 80) { $color = "yellow"; }
      else                              { $color = "lime"; }

      if ($WWW)
      {
        print "<tr bgcolor=$color>
            <td $style_normal> $this_drive
            <td $style_normal align=right> $this_drive_size
            <td $style_normal align=right> $this_drive_used
            <td $style_normal align=right> $this_drive_free
            <td $style_normal align=right> ${this_drive_pctused}%
        ";
      }
      else
      {
        &dataReturn("DRIVESTATUS: $this_drive $this_drive_size $this_drive_used $this_drive_free $this_drive_pctused");
      }
    }
    $last = $_;
  }


  if ($WWW)
  {
    $www_bgcolor = "pink";
    $www_bgcolor = "lime" if $numdbs > 0;

    #---- Exceptions
    $www_bgcolor = "pink" if $www_agent_status ne "Up";

    $www_status = "Down";
    $www_status = "Up" if $numdbs > 0;

    $daysup = $hoursup / 24;
    $daysup = sprintf("%.2f", $daysup);

    $mssql64bit_out = "64bit" if $mssql64bit;
    $os64bit_out = "64bit"    if $os64bit;

    &msg("WWW Status Summary:");
    print " <style type='text/css'>
	table.c1 {font-family:consolas; font-size:10pt; }
      </style> 
    <table border=0 bgcolor=wheat class=c1>
    <tr bgcolor=cadetblue>
      <th $style_normal>Item
      <th $style_normal>Status
    <tr bgcolor=$www_bgcolor>
      <td $style_normal>Hostname:
      <td $style_normal><b>$host</b>
    <tr bgcolor=$www_bgcolor>
       <td $style_normal>OS Version:
       <td $style_normal><b>$osver $os64bit_out $osname</b>
    <tr bgcolor=$www_bgcolor>
      <td $style_normal>MSSQL Instance Name:
      <td $style_normal><b>$in_instance</b>
    <tr bgcolor=$www_bgcolor>
      <td $style_normal>MSSQL Status:
      <td $style_normal><b>$www_status</b>
    <tr bgcolor=$www_bgcolor>
      <td $style_normal>MSSQL Service Status:
      <td $style_normal><b>$servicestat</b>
    <tr bgcolor=$www_bgcolor>
      <td $style_normal>MSSQL Agent Status:
      <td $style_normal><b>$www_agent_status</b>
    <tr bgcolor=$www_bgcolor>
      <td $style_normal>MSSQL Start Date:
      <td $style_normal><b>$mssql_started_fix</b>
    <tr bgcolor=$www_bgcolor>
      <td $style_normal>MSSQL Hours Active:
      <td $style_normal><b>$mssql_hours_up</b>
    <tr bgcolor=$www_bgcolor>
      <td $style_normal>Database Count:
      <td $style_normal><b>$numdbs</b>
    <tr bgcolor=$www_bgcolor>
      <td $style_normal>Database Space Alloc.:
      <td $style_normal><b>$tot_spacealloc mB</b>
    <tr bgcolor=$www_bgcolor>
      <td $style_normal>Database Space Used:
      <td $style_normal><b>$tot_spaceused mB</b>
    <tr bgcolor=$www_bgcolor>
      <td $style_normal>MSSQL Version:
      <td $style_normal><b>$ver $mssql64bit_out</b>
    "; 

    if ($cpu || $cpumodel) 
    {
      print "
      <tr bgcolor=$www_bgcolor>
        <td $style_normal>CPU Info:
        <td $style_normal>$cpu $cpumodel</b>
      "; 
    }
   
    print "
    <tr bgcolor=$www_bgcolor>
      <td $style_normal>CPU Count:
      <td $style_normal><b>$numcpus</b>
    <tr bgcolor=$www_bgcolor>
      <td $style_normal>RAM Total:
      <td $style_normal><b>$ram mB</b>
    <tr bgcolor=$www_bgcolor>
      <td $style_normal>RAM Available:
      <td $style_normal><b>$mem_avail mB</b>
    <tr bgcolor=$www_bgcolor>
      <td $style_normal>RAM Used:
      <td $style_normal><b>$mem_used mB</b>
    <tr bgcolor=$www_bgcolor>
      <td $style_normal>RAM % Used:
      <td $style_normal><b>${mem_pct}%</b>
    "; 

    if ($owmr_lr_age_days ne "") 
    {
      print "
      <tr bgcolor=$www_bgcolor>
	<td $style_normal>Server Uptime:
	<td $style_normal><b>$owmi_lr_age_days</b>
      ";
    }

    print "
    <tr bgcolor=$www_bgcolor>
      <td $style_normal>Remote Shell Service:
      <td $style_normal><b>$rsh_service</b>
    "; 

    if ($cluster_name ne "") 
    {
      print 
      "<tr bgcolor=$www_bgcolor>
	<td $style_normal>Cluster Name (If Any):
	<td $style_normal>$cluster_name
      <tr bgcolor=$www_bgcolor>
	<td $style_normal>Cluster Global Status (If Any):
	<td $style_normal>$cluster_status
      <tr bgcolor=$www_bgcolor>
	<td $style_normal>Cluster Active On Node (If Any):
	<td $style_normal>$cluster_node
      <tr bgcolor=$www_bgcolor>
	<td $style_normal>Cluster This-Node Status (If Any):
	<td $style_normal>$cluster_thisnodestatus
      "; 
    }
    
    if ($doubletake_status ne "") 
    {
      print "
	<tr bgcolor=$www_bgcolor>
	  <td $style_normal>DoubleTake Status:
	  <td $style_normal>$doubletake_status
	<tr bgcolor=$www_bgcolor>
	  <td $style_normal>DoubleTake Dir:
	  <td $style_normal>$doubletake_dir
	<tr bgcolor=$www_bgcolor>
	  <td $style_normal>DoubleTake File:
	  <td $style_normal>$doubletake_file
	<tr bgcolor=$www_bgcolor>
	  <td $style_normal>DoubleTake FileAge Mins:
	  <td $style_normal>$doubletake_fileage_mins
	"; 
    }

    print "
    </table><br>
    ";
  
    &msg("Databases:");
    print "<br><table border=0 bgcolor=wheat class=c1>\n";
    print "
    <tr bgcolor=cadetblue>
      <th $style_normal>Database<br>Name
      <th $style_normal>Alloc.<br>(mB)<br>DB+TLog
      <th $style_normal>Used<br>(mB)<br>DB+TLog
      <th $style_normal>Total Size (mB) Of DB File(s) 
      <th $style_normal>Total Size (mB) Of TLog File(s) 
      <th $style_normal>TLog Size to <br>DB Size <br>Ratio
      <th $style_normal>Date<br>Created
      <th $style_normal>Most Recent Object <br>Created Date
      <th $style_normal>Owner
      <th $style_normal>Datafile(s)
      <th $style_normal>Status
    \n";
    foreach (@www_dbs)
    {
      ##print "<tr bgcolor=$www_bgcolor>$_ \n";
      print "<tr bgcolor=white>$_ \n";
    }
    print "<tr bgcolor=cadetblue><td $style_normal><b> $fs Total:
      <td $style_normal align=right><b> $fs $tot_spacealloc
      <td $style_normal align=right><b> $fs $tot_spaceused 
      <td colspan=8> &nbsp;
    </table><br>";
  
  }
}

#------------------------------------------------------------------------------
sub runSPMonitor
#------------------------------------------------------------------------------
{
  &msg("runSPMonitor - Sampling CPU_BUSY from sp_monitor ");

  $sql = "exec sp_monitor";
  ##&msgl("Running DB=master sql=$sql");
  $sth = $dbh->prepare("$sql") || &error("Prepare");
  $sth->execute || &error("Execute_Select");
  while($href = $sth->fetchrow_hashref or $sth->{odbc_more_results})
  {
    # got what we needed, stop asking
    last if (defined $href->{cpu_busy});
  }
  $cpu_raw  = $href->{cpu_busy};
  $cpu_busy = (split(/\-/,$cpu_raw)) [1];
  $cpu_busy =~ s/\%//;
  &msg("runSPMonitor - Done - cpu_raw=$cpu_raw cpu_busy=$cpu_busy");
  $sth->finish || &error("Execute_FINISH");
}

#------------------------------------------------------------------------------
sub perfSample
#------------------------------------------------------------------------------
{
  $perfmode = $_[0];
  &msg("  perfSample - Starting - perfmode=$perfmode");

  #---- 1 Start. Total transaction count

  &msg("  Collecting TransRate sample"); 
  if ($vershort == 80) 
  {
    $sql = "select cntr_value from master.dbo.sysperfinfo 
      where counter_name = 'Transactions/sec' and instance_name = '_Total' "; 
  }
  else
  {
    $sql = "select cntr_value from master.sys.sysperfinfo 
      where counter_name = 'Transactions/sec' and instance_name = '_Total' "; 
  }

  $sth = $dbh->prepare("$sql") || &error("Prepare");
  $sth->execute || &error("Execute_Select");
  while(@Data = $sth->fetchrow_array)
  {
    $perf_all{"TransRate"} = $Data[0]; 
  }
  $sth->finish || &error("Execute_FINISH");

  #---- 1 End.   Total transaction count

  #---- 1 Start. Blocked sessions 
  $dbh->do("if OBJECT_ID('tempdb..#dbamon_spwho2') > 0 drop table #dbamon_spwho2") || &dbError("Drop_If_L1697");
  
  if ($vershort == 80) 
  {
    $dbh->do("create table #dbamon_spwho2 (
        SPID int not null
      , Status varchar (255) not null
      , Login varchar (255) not null
      , HostName varchar (255) not null
      , BlkBy varchar(10) not null
      , DBName varchar (255) null
      , Command varchar (255) not null
      , CPUTime int not null
      , DiskIO int not null
      , LastBatch varchar (255) not null
      , ProgramName varchar (255) null
      , SPID2 int not null
     )") || &dbError("Drop_If_L1697");
  }
  else
  {
    $dbh->do("create table #dbamon_spwho2 (
        SPID int not null
      , Status varchar (255) not null
      , Login varchar (255) not null
      , HostName varchar (255) not null
      , BlkBy varchar(10) not null
      , DBName varchar (255) null
      , Command varchar (255) not null
      , CPUTime int not null
      , DiskIO int not null
      , LastBatch varchar (255) not null
      , ProgramName varchar (255) null
      , SPID2 int not null
      , requestid int
     )") || &dbError("Drop_If_L1697");
  }
  $dbh->do("insert #dbamon_spwho2 exec sp_who2") || &dbError("Drop_If_L1697");
  
  $sql = "select COUNT(*)
    from #dbamon_spwho2
    where
    SPID > 50 and BlkBy != '  .  '
  "; 
  $sth = $dbh->prepare("$sql") || &error("Prepare");
  $sth->execute || &error("Execute_Select");
  while(@Data = $sth->fetchrow_array)
  {
    $perf_all{"BlockedSessions"} = $Data[0]; 
  }
  $sth->finish || &error("Execute_FINISH");
  
  $dbh->do("if OBJECT_ID('tempdb..#dbamon_spwho2') > 0 drop table #dbamon_spwho2") || &dbError("Drop_If_L1743");
 
  #---- 1 End.   Blocked sessions

  #---- Copy hash to the correct name
  if ($perfmode eq "Before") 
  { 
    %perf_before_all = %perf_all;
  }
  elsif ($perfmode eq "After") 
  {
    %perf_after_all = %perf_all;
  }
  else { &error("perfSample - Impossible perfmode=$perfmode") } 

  &msg("  perfSample - Done");
}

#------------------------------------------------------------------------------
sub SysPerfSample
#------------------------------------------------------------------------------
{
  $sps_counter = $_[0];
  $sps_label   = $_[1];
  &msg("  SysPerfSample - Starting - sps_counter=$sps_counter sps_label=$sps_label"); 
  $sps_file = "dbamon_typeperf_in.txt"; 
  $sps_val = "?"; 

  open(SPSFILE,">$sps_file") || &error("Cannot open output $sps_file - $!"); 
  print SPSFILE "$sps_counter\n"; 
  close(SPSFILE); 

  $sps_cmd = "typeperf.exe -cf $sps_file -sc 1"; 
  ##&msg("  Running sps_cmd=$sps_cmd"); 
  ##open(SPSCMD,"$sps_cmd|") || &error("Cannot run typeperf - $!"); 
  open(SPSCMD,"$sps_cmd|"); 
  while(<SPSCMD>)
  {
    next if /CSV/ || /^The command/ || /^Exiting/ || /^$/ || /^\s$/; 
    print;
    @sps_words = split (/\,/,$_) ; 
    if ($#sps_words == 1 && $sps_words[1] =~ /^\"/) 
    {
      $sps_val = $sps_words[1]; 
      $sps_val =~ s/\n//g;
    }
    last;
  }
  &msg("  Resulting sps_val=$sps_val"); 

  if ($sps_val eq "?") 
  {
    &msg("  Since not value was returned by TYPEPEF, returning prematurely"); 
    return;
  }

  $sps_val =~ s/\"//g; 
  $sps_val = sprintf("%.2f", $sps_val); 

  &msg("  SysPerfSample - Done - Displaying $sps_label $sps_val"); 
  &dataReturn("SYSPERF: $sps_label $sps_val"); 
  $sysperf{$sps_label} = $sps_val; 

  &msg("  SysPerfSample - Done"); 
  return; 
}

#------------------------------------------------------------------------------
sub SysPerf
#------------------------------------------------------------------------------
{
  &msg("SysPerf - Starting");
  $sp_start = time; 

  #---- Sample
  #typeperf "\MSSQL$MSSQL1:Buffer Manager\Buffer cache hit ratio" -sc 1
  
  #---- Each DB seems to have a different way of specifying the first part of the verb 
  #---- So by default we will use "SQLServer:", but we will query sys.dm_os_performance_counters to get the real value
  $counter_prefix = "SQLServer";   #---- Default
  if ($vershort > 80)
  {
    &connectDB if ! $dbh; 
    &msg("Selecting from dm_os_performance_counters to obtain counter_prefix"); 
    $sql = "select top 1 OBJECT_NAME from sys.dm_os_performance_counters";
    $sth = $dbh->prepare("$sql") || &dbError("Prepare1954");
    $sth->execute                || &dbError("Execute_Select1955");
    while (@Data = $sth->fetchrow_array)
    {
      $counter_string = $Data[0];
    }
    $sth->finish || &dbErrorOK("Finish1960");
    $counter_prefix_raw = (split(":",$counter_string)) [0];
    $counter_prefix = $counter_prefix_raw if $counter_prefix_raw;
    &msg("Resulting counter_string=$counter_string counter_prefix_raw=$counter_prefix_raw"); 
  }
  &msg("Using counter_prefix=$counter_prefix"); 

  #---- Now collect the data from the windoze performance counters

  #---- NEW - Collect from parms in CFG file
  &msg("From CFG file: \%cfg_perfcounter"); 
  foreach (sort keys %cfg_perfcounter)
  {
    $counter     = $_; 
    $dbamon_verb = $cfg_perfcounter{$_};

    if ($counter =~ /SQLSERVERPREFIX/) { $counter =~ s/SQLSERVERPREFIX/$counter_prefix/; } 
    
    &msg("  Found: counter=$counter dbamon_verb=$dbamon_verb - Invoking SysPerfSample");
    &SysPerfSample($counter, $dbamon_verb); 
  }

  #---- We have now collected all of the data

  if (1 == 0)
  {
    if (! $WWW) 
    {
      &msg("Resulting hash sysperf:"); 
      foreach $this_key (sort keys %sysperf)
      {
	&msg("  Key=$this_key Value=$sysperf{$this_key}");
      }
    }
  }
 
  if ($sysperf{"MemAvailMBytes"} > 0) 
  {
    $mem_avail = $sysperf{"MemAvailMBytes"} + 0;  
    &msg("Set mem_avail from sysperf hash - mem_avail=$mem_avail"); 
  } 
  
  $sp_elapsed = time - $sp_start; 
  &msg("SysPerf - Done - Elapsed: $sp_elapsed second(s)");
}

#------------------------------------------------------------------------------
sub SysPerfBulk
#------------------------------------------------------------------------------
{
  &msg("SysPerfBulk - Starting - typeperf_sample_dur=$typeperf_sample_dur");
  $spb_stime = time; 

  #---- Each DB seems to have a different way of specifying the first part of the verb 
  #---- So by default we will use "SQLServer:", but we will query sys.dm_os_performance_counters to get the real value
  $counter_prefix = "SQLServer";   #---- Default
  undef %sysperf_bulk;
  undef %perf_counters;
  my $perf_probeout; 
  $debug_perfbulk = 0; 

  if ($vershort > 80)
  {
    &connectDB if ! $dbh; 
    &msg("Selecting from dm_os_performance_counters to obtain counter_prefix"); 
    $sql = "select top 1 OBJECT_NAME from sys.dm_os_performance_counters";
    $sth = $dbh->prepare("$sql") || &dbError("Prepare1954");
    $sth->execute                || &dbError("Execute_Select1955");
    while (@Data = $sth->fetchrow_array)
    {
      $counter_string = $Data[0];
    }
    $sth->finish || &dbErrorOK("Finish1960");
    $counter_prefix_raw = (split(":",$counter_string)) [0];
    $counter_prefix = $counter_prefix_raw if $counter_prefix_raw;
    &msg("Resulting counter_string=$counter_string counter_prefix_raw=$counter_prefix_raw"); 
  }
  &msg("Using counter_prefix=$counter_prefix"); 

  #---- Now collect the data from the windoze performance counters

  #---- NEW - Collect from parms in CFG file - Write all out to $sps_file
  $sps_file_in  = "dbamon_typeperf_in.txt"; 
  open(SPSFILE,">$sps_file_in") || &error("Cannot open output $sps_file_in - $!"); 

  &msg("Writing Perf Counters from CFG file to $sps_file_in"); 
  foreach (sort keys %cfg_perfcounter)
  {
    $counter     = $_; 
    $dbamon_verb = $cfg_perfcounter{$_};

    if ($counter =~ /SQLSERVERPREFIX/) { $counter =~ s/SQLSERVERPREFIX/$counter_prefix/; } 
    
    &msg("  Found: counter=$counter dbamon_verb=$dbamon_verb - Writing to $sps_file_in") if $debug_perfbulk;
    print SPSFILE "$counter\n"; 
    $counter =~ s/^\\//; 
    $perf_counters{$counter} = $dbamon_verb;
  }
  close(SPSFILE); 

  #---- Decide which command to use - typeperf or dbamon_oswin_tpr 
  $typeperf_cmd = "typeperf.exe"; 
  &msg("Verifying that typeperf exists"); 
  open(SPSCMD,"$typeperf_cmd 2>&1 |"); 
  while(<SPSCMD>)
  {
    if (/is not recognized as an internal or external command/i || -e "Override_TPE.txt")
    {
      $typeperf_cmd = "dbamon_oswin_tpe.exe";
      &msg("  typeperf NG - Will use typeperf_cmd=$typeperf_cmd");
      last;
    } 
  }
  close(SPSCMD); 
  &msg("We will be using typeperf_cmd=$typeperf_cmd"); 

  #---- Run typeperf and collect the results
  $sps_first_line = 0; 

  #---- Old
  ##$sps_cmd = "$typeperf_cmd -cf $sps_file_in -sc 1 -si $typeperf_sample_dur"; 
  #---- New
  $sps_cmd = "$typeperf_cmd -cf $sps_file_in -sc 1"; 

  ##&msg("Running $typeperf_sample_dur second sample of performance counters - sps_cmd=$sps_cmd"); 
  &msg("Running sample of performance counters - sps_cmd=$sps_cmd"); 

  open(SPSCMD,"$sps_cmd 2>&1 |"); 
  while(<SPSCMD>)
  {
    next if /^\s$/; 
    next if /^Warning:/; 
    print;
    
    @sps_words = split (/\,/,$_) ; 
    if (! $sps_first_line && /CSV /) 
    {
      $icnt = 0; 
      foreach $this_word (@sps_words)
      {
        $this_word =~ s/\n//g; 
        $this_counter = $this_word; 
        $this_counter =~ s/\\\\$host_uc\\//; 
        $this_counter =~ s/\"//g; 
        $icnt++; 
        $typeperf_keys{$icnt} = $this_counter; 
        &msg("  DEBUG - COUNTER - icnt=$icnt this_word=$this_word this_counter=$this_counter") if $debug_perfbulk; 
      }
      $sps_first_line = 1; 
      $field_cnt_line_1 = $icnt;
      next; 
    }
    else   #---- There should just be 2 non-null lines
    {
      $icnt = 0; 
      foreach $this_word (@sps_words)
      {
        next if $this_word =~ /CSV /; 

        $this_word =~ s/\n//g; 
        $this_word =~ s/\"//g; 
        #------------------------------------------------------------------------
        # 2.27 - 2012/04/07 BB: Strange problem found by City of COS. They had
        #   some servers where missing perf counters (not found on that server)
        #   had OK FIELD NAMES but every missing counter showed a -1 in the value
        #   line. So, this line of code will skip over all values that are -1 in 
        #   such a way that we don't increment $icnt, which should make the number
        #   of values equal to the number of column names in the previous line. 
        #   Strange!!!
        # 
        next if $this_word == -1;     #--- Skip bad values and get back in sync
        #------------------------------------------------------------------------
        $icnt++; 
        $this_key         = $typeperf_keys{$icnt}; 
        $this_dbamon_verb = $perf_counters{$this_key}; 
        &msg("  DEBUG - VAL - icnt=$icnt this_word=$this_word this_key=$this_key this_dbamon_verb=$this_dbamon_verb") if $debug_perfbulk; 
        next if ! $this_dbamon_verb; 

        #---- Processing for counters with many decimal places
        if ($this_dbamon_verb =~ /MULT1000$/)
        {
          $this_dbamon_verb =~ s/MULT1000//; 
          $this_word = $this_word * 1000; 
        }
     
        $perf_probeout .= "$P SYSPERF: $this_dbamon_verb $this_word\n"; 
        $sysperf_bulk{$this_dbamon_verb} = $this_word; 
        $sysperf_bulk{$this_dbamon_verb} = sprintf("%.2f", $sysperf_bulk{$this_dbamon_verb}); 
      }
      $field_cnt_line_2 = $icnt;
      last; 
    }
  }
  close(SPSCMD);

  #---- We have now collected all of the data - Compare field counts
  $field_cnt_line_1 += 0;
  $field_cnt_line_2 += 0;
  &msg("field_cnt_line_1=$field_cnt_line_1"); 
  &msg("field_cnt_line_2=$field_cnt_line_2"); 

  if ($field_cnt_line_1 == $field_cnt_line_2) 
  {
    &msg("Field counts are equal - Good!"); 
  }
  else
  {
    ##&msg("PHONEHOME: SysPerfBulk_Unequal_Field_Counts_LINE1=${field_cnt_line_1}_LINE2=${field_cnt_line_2}"); 
    &dataReturn("SYSPERF: BadCounterValues 1"); 
    return; 
  }

  my $sp_count_nonzero = 0; 

  if (! $WWW) 
  {
    &msg("Resulting hash sysperf_bulk:"); 
    foreach $this_key (sort keys %sysperf_bulk)
    {
      &msg("  Key=$this_key Value=$sysperf_bulk{$this_key}");
      $sp_count_nonzero++ if $sysperf_bulk{$this_key} > 0;   
    }
  }

  &msg("SysPerfBulk - Count of non-zero values: $sp_count_nonzero"); 
  &msg("PHONEHOME: SysPerfBulk - Count_of_non-zero_values=${sp_count_nonzero}_(too_few)") if $sp_count_nonzero < 1 && $osname ne "Win2000"; 

  #---- Write out the counters that we collected
  if ($perf_probeout) 
  { 
    &msg("PERF Probeout:\n$perf_probeout") unless $WWW; 
  }
 
  if ($sysperf_bulk{"MemAvailMBytes"} > 0) 
  {
    $mem_avail = $sysperf_bulk{"MemAvailMBytes"} + 0;  
    &msg("Set mem_avail from sysperf_bulk hash - mem_avail=$mem_avail"); 
  } 
  
  $spb_elapsed = time - $spb_stime; 
  &msg("SysPerfBulk - Done - Elapsed: $spb_elapsed second(s)");
}

#------------------------------------------------------------------------------
sub SysPerfSpecial
#------------------------------------------------------------------------------
{
  &msg("SysPerfSpecial - Starting");

  #---- #1 - Blocked processes GROSS
  $sql = "select count(*) from master..sysprocesses (nolock) where blocked != 0"; 
  $sth = $dbh->prepare($sql) || &dbError("Prepare_L2556");
  $sth->execute || &dbErrorOK("Execute_L2557");
  while (@Data = $sth->fetchrow_array)
  {
    &dataReturn("SYSPERF: SQL_BlockedProcsGross $Data[0]"); 
  }
  
  #---- #2 - Blocked processes BLOCKED BY SOMEONE ELSE
  $sql = "select count(*) from master..sysprocesses (nolock) where blocked != 0 and blocked != spid"; 
  $sth = $dbh->prepare($sql) || &dbError("Prepare_L2565");
  $sth->execute || &dbErrorOK("Execute_L2566");
  while (@Data = $sth->fetchrow_array)
  {
    &dataReturn("SYSPERF: SQL_BlockedProcsOtherProc $Data[0]"); 
  }

  #---- #3 - Check for Lock Pages In Memory WORKING (2008 and higher only) 
  if ($vershort >= 100)
  {
    $sql = "select locked_page_allocations_kb from master.sys.dm_os_process_memory"; 
    $sth = $dbh->prepare($sql) || &dbError("Prepare_L2613");
    $sth->execute || &dbErrorOK("Execute_L2614");
    while (@Data = $sth->fetchrow_array)
    {
      &dataReturn("SYSPERF: SQL_LPIM_Locked_KB $Data[0]"); 
    }
  }

  #---- #4 - RUNNABLE Requests
  if ($vershort >= 90)
  {
    $sql = "select count(*) from master.sys.dm_exec_requests where status = 'runnable'"; 
    $sth = $dbh->prepare($sql) || &dbError("Prepare_L2624");
    $sth->execute || &dbErrorOK("Execute_L2624");
    while (@Data = $sth->fetchrow_array)
    {
      &dataReturn("SYSPERF: SQL_Runnable_Reqs $Data[0]"); 
    }
  }

  #---- #4A - RUNNABLE Tasks (different)
  if ($vershort >= 90)
  {
    $sql = "SELECT SUM(runnable_tasks_count) AS sum_runnable_tasks FROM sys.dm_os_schedulers WHERE scheduler_id < 255"; 
    $sth = $dbh->prepare($sql) || &dbError("Prepare_L2624");
    $sth->execute || &dbErrorOK("Execute_L2624");
    while (@Data = $sth->fetchrow_array)
    {
      &dataReturn("SYSPERF: SQL_Runnable_Tasks $Data[0]"); 
    }
  }
  #---- #5A - TOP WAITs for SQL 2005+
  if ($vershort >= 90)
  {
    $sql = "
      WITH    Waits
          AS ( SELECT TOP 5
                        wait_type ,
                        wait_time_ms / 1000. AS wait_time_s ,
                        100. * wait_time_ms / SUM(wait_time_ms) OVER ( ) AS pct ,
                        ROW_NUMBER() OVER ( ORDER BY wait_time_ms DESC ) AS rn
               FROM     sys.dm_os_wait_stats WITH (NOLOCK)
               WHERE    wait_type NOT IN ( 'CLR_SEMAPHORE', 'LAZYWRITER_SLEEP',
                                           'RESOURCE_QUEUE', 'SLEEP_TASK',
                                           'SLEEP_SYSTEMTASK',
                                           'SQLTRACE_BUFFER_FLUSH', 'WAITFOR',
                                           'LOGMGR_QUEUE', 'CHECKPOINT_QUEUE',
                                           'REQUEST_FOR_DEADLOCK_SEARCH',
                                           'XE_TIMER_EVENT', 'BROKER_TO_FLUSH',
                                           'BROKER_TASK_STOP',
                                           'CLR_MANUAL_EVENT',
                                           'CLR_AUTO_EVENT',
                                           'DISPATCHER_QUEUE_SEMAPHORE',
                                           'FT_IFTS_SCHEDULER_IDLE_WAIT',
                                           'XE_DISPATCHER_WAIT',
                                           'XE_DISPATCHER_JOIN',
                                           'SQLTRACE_INCREMENTAL_FLUSH_SLEEP',
                                           'ONDEMAND_TASK_QUEUE',
                                           'BROKER_EVENTHANDLER',
                                           'SLEEP_BPOOL_FLUSH',
                                           'HADR_FILESTREAM_IOMGR_IOCOMPLETION',
                                           'SP_SERVER_DIAGNOSTICS_SLEEP',
                                           'DIRTY_PAGE_POLL',
                                           'BROKER_TRANSMITTER' )
                        AND wait_type NOT LIKE 'BACKUP%'
             )
    SELECT  W1.wait_type ,
            CAST(W1.wait_time_s AS DECIMAL(12, 2)) AS wait_time_s ,
            CAST(W1.pct AS DECIMAL(12, 2)) AS pct ,
            CAST(SUM(W2.pct) AS DECIMAL(12, 2)) AS running_pct
    FROM    Waits AS W1
            INNER JOIN Waits AS W2 ON W2.rn <= W1.rn
    GROUP BY W1.rn ,
            W1.wait_type ,
            W1.wait_time_s ,
            W1.pct
    "; 
    $sth = $dbh->prepare($sql) || &dbError("Prepare_L2808");
    $sth->execute || &dbErrorOK("Execute_L2809");
    $perf_rank = 0; 
    while (@Data = $sth->fetchrow_array)
    {
      $perf_rank++; 
      &dataReturn("SYSPERF: SQL_Top_Waits_Rank$perf_rank $Data[0] $perf_rank $Data[2]"); 
    }
  }

  #---- #5B - TOP WAITs for SQL 2000
  ## Testing -- if ($vershort > 90)
  if ($vershort < 90)
  {
    &SysPerf2000Waits;
  }

  #---- #6 - Week Old SQL Session
  if ($vershort >= 90)
  {
    $sql = "
SELECT  COUNT(*) AS WeekOldSesCount
FROM    master..sysprocesses
WHERE   spid > 50
        AND DB_NAME(dbid) != 'msdb'
        AND loginame NOT LIKE '%ntservice%'
        AND program_name NOT LIKE '%jobstep%'
		AND DATEDIFF(dd, login_time, GETDATE()) >= 7
    "; 
    $sth = $dbh->prepare($sql) || &dbError("Prepare_L2967");
    $sth->execute || &dbErrorOK("Execute_L2968");
    while (@Data = $sth->fetchrow_array)
    {
      &dataReturn("SYSPERF: SQL_Sessions_Week_Old $Data[0]"); 
    }
  }

  #---- Return
  &msg("SysPerfSpecial - Done");
}

#------------------------------------------------------------------------------
sub SysPerf2000Waits
#------------------------------------------------------------------------------
{
  &msg("SysPerf2000Waits - Starting");

  # /*
  # From: http://www.sql-server-pro.com/get-wait-stats.html
  # */
  #  
  # set nocount on;
  # create table #wt (types varchar(30))
  #  
  # -- Add any wait types you want to ignore here. 'Total' MUST be ignored.
  # insert into #wt values('Total')
  # insert into #wt values('LAZYWRITER_SLEEP')
  # insert into #wt values('SLEEP')
  # insert into #wt values('OLEDB')
  # insert into #wt values('RESOURCE_QUEUE')
  # insert into #wt values('WAITFOR')
  #  
  # create table #ws (wait_type varchar(255),
  #	      requests decimal(20, 0),
  #	      wait_time_ms decimal(20, 0),
  #	      signal_wait_time decimal(20, 0))
  #    
  #   insert #ws exec ('dbcc sqlperf(waitstats) with no_infomsgs')
  #      
  #   select top 5 wait_type,
  #   cast(wait_time_ms / 1000. as decimal(12, 2)) wait_time_s,
  #   cast(100. * wait_time_ms /
  #	   (select sum(wait_time_ms)
  #	    from #ws
  #	    where wait_type not in
  #		    (select types from #wt)) as decimal(12, 2)) pct,
  #   case when requests > 0
  #	then cast((wait_time_ms - signal_wait_time) / requests as int)
  #	else 0 end avg_wait_ms
  #   from #ws
  #   where wait_type not in (select types from #wt)
  #               by 3 desc
  #    
  #   drop table #ws
  #   drop table #wt

  my $not_in_list = "'Total', 'LAZYWRITER_SLEEP', 'SLEEP', 'OLEDB', 'RESOURCE_QUEUE', 'WAITFOR'"; 

  $dbh->do("create table #dbamon_ws (
    wait_type        varchar(255),
    requests         decimal(20, 0),
    wait_time_ms     decimal(20, 0),
    signal_wait_time decimal(20, 0)
    )") || &dbError("CT_2870");

  ## $dbh->do("insert #dbamon_ws exec ('dbcc sqlperf(waitstats) with no_infomsgs'") || &dbError("INS_2874"); 
  ## $sql = "exec ('DBCC SQLPERF (LOGSPACE)')";

  $dbh->do("use master") || &dbError("USE_2876"); 
  $dbh->do("INSERT #dbamon_ws EXEC ('DBCC SQLPERF (WAITSTATS)')") || &dbError("INS_2874"); 

  $sql = "select top 5 
    wait_type,
    cast(wait_time_ms / 1000. as decimal(12, 2)) wait_time_s,
    cast(100. * wait_time_ms /
      (select sum(wait_time_ms)
         from #dbamon_ws
         where wait_type not in ($not_in_list) 
      ) as decimal(12, 2)) pct,
    case 
      when requests > 0 then cast((wait_time_ms - signal_wait_time) / requests as int)
      else 0 
    end avg_wait_ms
  from #dbamon_ws
  where 
    wait_type not in ($not_in_list)
  order by 
    3 desc
  ";
  $sth = $dbh->prepare("$sql") || &dbError("PRE_2894");
  $sth->execute || &dbError("EXE_2895");

  my $perf_rank = 0; 

  while (@Data = $sth->fetchrow_array)
  {
    $perf_rank++; 
    &dataReturn("SYSPERF: SQL_Top_Waits_Rank$perf_rank $Data[0] $perf_rank $Data[2]"); 
  }

  $dbh->do("drop table #dbamon_ws") || &dbError("DRP_2876"); 

  &msg("SysPerf2000Waits - Done");
}

#------------------------------------------------------------------------------
sub SysPerfCompare
#------------------------------------------------------------------------------
{
  &msg("SysPerfCompare - Starting");
  my $zstring = ""; 
  my $zatleastone = 0; 

  &msg("Resulting hash sysperf:"); 
  foreach $this_key (sort keys %sysperf)
  {
    $zstring = sprintf("%-40s %12.2f", $this_key, $sysperf{$this_key});  
    &msg("  $zstring");
  }

  &msg("Resulting hash sysperf_bulk:"); 
  foreach $this_key (sort keys %sysperf_bulk)
  {
    $zstring = sprintf("%-40s %12.2f", $this_key, $sysperf_bulk{$this_key});  
    &msg("  $zstring");
    $zatleastone = 1 if $sysperf_bulk{$this_key} > 0;  
  }

  &msg("Timing:"); 
  &msg("  Elapsed BULK:  $spb_elapsed"); 
  &msg("  Elapsed Other: $sp_elapsed"); 

  &msg("Key Count:"); 
  my $count = keys %sysperf_bulk;
  my $countb = keys %sysperf;
  &msg("  KeyCount BULK:  $countb"); 
  &msg("  KeyCount Other: $count"); 

  if ($count == $countb && $zatleastone)
  {
    &msg("Everything looks GOOD - Equal key counts and at least 1 Bulk value - Copying sysperf_bulk to sysperf");   
    $sysperf = $sysperf_bulk; 
  }  
  elsif  ($count >= 3 && $countb >= 3)
  {
    &msg("Everything looks GOOD - Bulk and Other each have >= 3 keys - Copying sysperf_bulk to sysperf");   
    $sysperf = $sysperf_bulk; 
  }  
  elsif ($count == 0 && $countb == 0)
  {
    &msg("Everything looks GOOD - ?? both Bulk and Other each have 0 keys (I guess this is OK?)  - Copying sysperf_bulk to sysperf");   
    $sysperf = $sysperf_bulk; 
  }  
  else
  {
    &msg("PHONEHOME: Something_wrong-Sysperf_element_count_!=_SysperfBULK"); 
  }

  &msg("SysPerfCompare - Done");
}

#------------------------------------------------------------------------------
sub SQLPerf
#------------------------------------------------------------------------------
{
  $perf_before_time = time; 
  $perf_interval = 5; 

  &msg("SQLPerf - Starting - perf_before_time=$perf_before_time");

  #---- Call perfSample BEFORE
  &perfSample("Before"); 
 
  #---- Sleep before taking 2nd snapshot
  &msg("  Sleeping perf_interval=$perf_interval"); 
  sleep $perf_interval; 

  #---- Call perfSample AFTER
  $perf_after_time = time; 
  &perfSample("After"); 

  #---- Calculations
  $perf_el_time = $perf_after_time - $perf_before_time;
  &msg("  SQLPerf - Calculating Deltas - perf_before_time=$perf_before_time perf_after_time=$perf_after_time perf_el_time=$perf_el_time");

  foreach $p_metric (sort keys %perf_after_all)
  {
    $p_before_val = $perf_before_all{$p_metric};
    $p_after_val  = $perf_after_all{$p_metric};
    $p_delta_val  = $p_after_val - $p_before_val; 
    $p_per_second = -1;  #---- In case of zero divide - Unlikely
    $p_per_second = sprintf("%.2f", ($p_delta_val / $perf_el_time)) if $perf_el_time > 0;

    if ($p_metric =~ /rate/i) 
    {
      &msg("  Metric=$p_metric p_before_val=$p_before_val p_after_val=$p_after_val p_delta_val=$p_delta_val p_per_second=$p_per_second");  
      &dataReturn("PERF: $p_metric $p_per_second"); 
    }
    else
    {
      &msg("  Metric=$p_metric p_before_val=$p_before_val p_after_val=$p_after_val");  
      if ($p_before_val > $p_after_val) { $p_final_val = $p_before_val; } 
      else                              { $p_final_val = $p_after_val; } 
      &dataReturn("PERF: $p_metric $p_final_val"); 
    }
  }

  #$perf_total_trans = $perf_after_transcount - $perf_before_transcount; 
  #$perf_total_transrate = sprintf("%.2f", ($perf_total_trans / $perf_el_time)) if $perf_el_time > 0;

  #&msg("SQLPerf - BEFORE - perf_before_time=$perf_before_time - perf_before_transcount=$perf_before_transcount");
  #&msg("SQLPerf - AFTER  - perf_el_time=$perf_el_time - perf_after_transcount=$perf_after_transcount");

  &msg("SQLPerf - Done");
}

# ----------------------------------------------------------------------
sub bkrun
# ----------------------------------------------------------------------
{
  $db     = $arg1;
  $method = $arg2;
  $cmd    = $arg3;

  &msg("bkrun - Running backup for DB=$db Method=$method");
  &msg("bkrun: Command before {DB}, {SQLPATH}, {PERLPATH} and backslash substitution: $cmd");

  # 24Jan04 - Now PERLPATH contains the path to the executable
  #  If they specified perl.exe in the CMD, that is wrong. 
  if ($cmd =~ /perl.exe/ && $cmd =~ /PERLPATH/)
  {
    &msg("NOTE: PERLPATH is wrong - It specifies perl.exe - modifying"); 
    $cmd =~ s/\/perl.exe//g;
    &msg("Modified Command after substitution: $cmd");
    $modified = 1;
  }

  $cmd =~ s/\{DB\}/\"$db\"/g;
  $cmd =~ s'\[BLANK\]' 'g;
  $cmd =~ s'\[HYPHEN\]'-'g;

  $cmd =~ s/\{SQLPATH\}/$SQLPATH/g;
  $cmd =~ s/\{PERLPATH\}/$PERLPATH/g;

  $cmd =~ s/\//\\/g;
  &msg("bkrun: Command after substitution: $cmd");

  #DEBUG START
  system("$PERLPATH C:\\dbatools\\socketmail.pl dbaadmin\@dba.it.thecompany.com dbaadmin\@dba.it.thecompany.com \"dbamon_oswin.pl Modified_CMD   PERLPATH=$PERLPATH host=$host modified=$modified cmd=$cmd [AUTO]\" $dbname/$backup_type") if $modified;

  system("$PERLPATH C:\\dbatools\\socketmail.pl dbaadmin\@dba.it.thecompany.com dbaadmin\@dba.it.thecompany.com \"dbamon_oswin.pl PERLPATH_Not_E PERLPATH=$PERLPATH host=$host modified=$modified cmd=$cmd [AUTO]\" $dbname/$backup_type") if ! -e $PERLPATH;
  #DEBUG END 

  if ($method eq "B") { $syscmd = "start/b \"MSSQL $db Backup\" $cmd"; }
  else                { $syscmd = "$cmd"; }

  &msg("$bkrun: About to issue: $syscmd");
  $s_time = time;
  $rc = system($syscmd);
  $e_time = time;
  $el_time = $e_time - $s_time;
  &msg("bkrun: Back from issuing: $syscmd - rc=$rc el_time=$el_time");

  $rc /= 256 if $rc != 0;
  if ($rc != 0)
  {
    &msg("ERROR: Invoked Backup_Command: ended with non-zero rc=$rc");
  }
}

# ----------------------------------------------------------------------
sub bkhi
# ----------------------------------------------------------------------
{
  $dbname = $arg1; 

  $dbname =~ s/\[HYPHEN\]/\-/g;
  $dbname =~ s/\[BLANK\]/\ /g;
  
  # use strict;
  use DBI;

  # Attempt to connect to the master database on the server, to verify MSSQL availability.
  &connectDB;
  
  $dbh->do("use msdb")         || &dbError("Use_MSDB");

  if (!$dberror)
  {
    # Create a report of all backups for this DB
    ##if ($ver =~ "6.5")
    if ($vershort < 70)
    { 
      # Check backup age for this db
      $sql = "
       select
	  substring(database_name,1,16) as db,
	  backup_type,
	  backup_finish,
	  datediff(hour,backup_finish, getdate()) as hours_since
       from
	  sysbackuphistory 
       where
	  backup_type = 1 
	  and database_name = '$dbname'
	  and datediff(day, backup_finish, getdate()) <= 50
       order by
	  3 desc";
    }
    else
    {
      # Check backup age for this db
      $sql = "
	 select
	    substring(database_name,1,16) as db,
	    type,
	    backup_finish_date,
	    datediff(hour, backup_finish_date, getdate()) as hours_since
	 from
	    backupset
	 where
	    database_name = '$dbname'
	    --and datediff(day, backup_finish_date, getdate()) <= 50
	 order by
	    3 desc";
    }

    &msg("Running query of MSDB for Backup History Report for database $dbname");
    $sth = $dbh->prepare("$sql") 
      || &dbError("Prepare");
    $sth->execute
      || &dbError("Execute_Select");

    print "\n<table border=1 bgcolor=skyblue class=c1>
    <tr bgcolor=slate> 
      <th><font size=2>Backup<br>Type
      <th><font size=2>Timestamp
      <th><font size=2>Age<br>(Hours)
    \n";
    $count = 0;
    while (@Data = $sth->fetchrow_array)
    {
      ++$count;
      $db      = $Data[0];
      $bcktype = $Data[1];
      if ($bcktype eq "1") { $bcktype = "D" }; 
      $bckts   = $Data[2];
      $bckage  = $Data[3];

      if    ($bcktype eq "D") { $bt_out = "Database"; }
      elsif ($bcktype eq "I") { $bt_out = "Database Differential"; }
      elsif ($bcktype eq "L") { $bt_out = "Log"; }
      elsif ($bcktype eq "F") { $bt_out = "File"; }
      else                        { $bt_out = "?"; }

      $bt = "($bcktype)$bt_out";

      printf "<tr>
        <td><font size=2>%-4s </font>
        <td><font size=2>%-24s </font>
        <td align=right><font size=2>%-6s</font>
      \n",
      $bt,
      $bckts,
      $bckage
      ;
      $backup_count++;
    }
    print "</table>\n";
    print "\n<b>Total Backups: $backup_count</b>\n\n"; 
  }
}

# ----------------------------------------------------------------------
sub oldPerl
# ----------------------------------------------------------------------
{
  &msg("oldPerl - Starting - Looking for residual Perl processes"); 

  $pslist_loc = "C:\\dbatools\\pslist.exe";
  $oldperl_t  = 24;    #---- 1.27
  $oldperl_cnt = 0;
 
  if (! -e $pslist_loc)
  {
    &msg("pslist is not found at $pslist_loc - Skipping");
    return;
  }

  &msg("Looking for old Perl procesess that have been running for at least $oldperl_t hours under userid=$userid - pslist_loc=$pslist_loc");

  $cmd = "$pslist_loc > NUL 2>&1";
  &msg("Testing $pslist_loc to make sure that it works on this server - cmd=$cmd");
  if (system("$cmd") != 0)
  {
    &msg("pslist Error encountered - Skipping");
    return;
  }

  open(CMD, "$pslist_loc |") || &error("Cannot invoke $pslist_loc - $!");
  while (<CMD>)
  {
    @words  = split;
    $pname  = $words[0];
    $pid    = $words[1];
    $utime  = $words[6];
    $ktime  = $words[7];
    $etime  = $words[8];

    $hours = (split(":",$etime)) [0];

    next if $pname !~ /Perl/;
    next if $hours < $oldperl_t;

    # Get username
    open(CMD2, "pulist  |") || print "$P | Cannot invoke $pslist_loc - $!\n";;
    while (<CMD2>)
    {
      @words  = split;
      $pgm2   = $words[0];
      $pid2   = $words[1];
      $usr2   = $words[2];

      ##print "pgm2=$pgm2 pid2=$pid2 usr2=$usr2\n"; 
 
      next if $pid2 ne $pid;

      $user   = $words[2];
    }
    close(CMD2);

    next if index($user,$userid) == -1;

    print "$P OLDPERL: pid=$pid pname=$pname etime=$etime hours=$hours user=$user utime=$utime ktime=$ktime\n";
    $oldperl_cnt++;
  }
  close(CMD);

  &msg("oldPerl - Done - Found $oldperl_cnt old perl processes"); 
}

# ----------------------------------------------------------------------
sub checkDoubleTake
# ----------------------------------------------------------------------
{
  &msg("checkDoubleTake - Starting - Checking DoubleTake (registry)"); 

  #---- See if it is installed
  %values="";
  $Register = "Software\\NSI Software\\Double-Take\\CurrentVersion";
  $HKEY_LOCAL_MACHINE->Open($Register,$hkey); 
  ##&msg("  hkey=$hkey"); 
  if ($hkey eq "")
  {
    &msg("  DoubleTake is NOT installed"); 
    return;
  }
  $hkey->GetValues(\%values) || &msg("  Cannot find DoubleTake in registry");
  foreach $value (keys(%values))
  {
    $RegType      = $values{$value}->[1];
    $RegValue     = $values{$value}->[2];
    $RegKey       = $values{$value}->[0];
    $doubletake_version = $RegValue if $RegKey eq "VersionInfo";
  }
  $hkey->Close();
  if ($doubletake_version eq "")
  {
    &msg("  DoubleTake is NOT installed"); 
    return;
  }
  &msg("  DoubleTake version $doubletake_version is installed"); 

  #---- Get MSSQL base dir
  %values="";
  $Register = "Software\\Microsoft\\MSSQLServer\\MSSQLServer\\Parameters";
  $HKEY_LOCAL_MACHINE->Open($Register,$hkey); 
  $hkey->GetValues(\%values);
  foreach $value (keys(%values))
  {
    $RegType       = $values{$value}->[1];
    $RegValue      = $values{$value}->[2];
    $RegKey        = $values{$value}->[0];
    $mssql_basedir = substr($RegValue,2) if $RegKey eq "SQLArg0";
  }
  $hkey->Close();

  $zraw = $mssql_basedir;
  $zlookfor = "\\"; 
  $zidx = rindex($zraw,$zlookfor); 
  $doubletake_dir = substr($zraw,0,$zidx);

  &msg("  mssql_basedir=$mssql_basedir doubletake_dir=$doubletake_dir"); 

  $doubletake_fileage_mins = 999999; 
  $doubletake_file = "$doubletake_dir\\DBAmon_Timestamp.txt"; 
  if (-e $doubletake_file) 
  {
    open(TS, "$doubletake_file") || print "$P | Cannot open input $doubletake_file - $!\n";;
    while (<TS>)
    {
      @words = split;
      $tsnum = $words[0];
    }
    $doubletake_fileage_mins = int((time - $tsnum) / 60); 
  }

  #---- See if doubletake is running
  $doubletake_status = "?";
  open(NET, 'net start|') || print "Error: Running net start";
  while(<NET>)
  {
    ##print;
    if (/Double-Take/)            { $doubletake_status="[Running]"; }
  }
  close(NET);

  &dataReturn("DoubleTakeStatus:      $doubletake_status");
  &dataReturn("DoubleTakeDir:         $doubletake_dir");
  &dataReturn("DoubleTakeFile:        $doubletake_file");
  &dataReturn("DoubleTakeFileAgeMins: $doubletake_fileage_mins");

  &msg("checkDoubleTake - Done"); 
}

# ----------------------------------------------------------------------
sub connectDBOld
# ----------------------------------------------------------------------
{
  &msg("connectDB() Connecting to ODBC driver=$driver as 'guest'"); 
  $dbh = DBI->connect("DBI:ODBC:$driver",'guest','' ) || &dbError("Connect_To_DB");

  #if ($host eq "wbbndba1")
  #{
  #  &msg("  Setting isolation level: SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED"); 
  #  $dbh->do("SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED") || &dbError("SET ISO LEVEL");
  #}
  &msg("  Setting isolation level: SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED"); 
  $dbh->do("SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED") || &dbError("SET ISO LEVEL");
}

# ----------------------------------------------------------------------
sub connectDB
# ----------------------------------------------------------------------
{
  &msg("connectDB() Connecting to ODBC without driver (dynamic)"); 

  if ($default_instance) 
  {
    if (-e "dbamon_oswin_Parm_ConnectToHostname.txt")
    {
      $DSN = "driver={SQL Server};Server=$host;uid=;pwd=;"; 
    }
    else
    {
      $DSN = "driver={SQL Server};Server=localhost;uid=;pwd=;"; 
    }
  }
  else
  {
    #---- OLD: if ($vershort <= 80) 
    ## if ($vershort <= 80 || -e "dbamon_oswin_Parm_ConnectToHostname.txt")
    #---- Some servers don't have a localhost DNS entry - this touch file is for that
    if (-e "dbamon_oswin_Parm_ConnectToHostname.txt")
    {
      $DSN = "driver={SQL Server};Server=$host\\$in_instance;uid=;pwd=;"; 
    }
    else
    {
      $DSN = "driver={SQL Server};Server=localhost\\$in_instance;uid=;pwd=;"; 
    }
  }

  if ($dbh ne "") 
  {
    &msg("Already connected to DB - dbh is not null"); 
    return;
  }

  &msg("  connecting DSN=$DSN") ;
  $dbh = DBI->connect("dbi:ODBC:$DSN", 
  { PrintError => 1,
    RaiseError => 0 } ) || &error("DBConnect");

  #---- Set long read len
  $dbh->{LongReadLen} = 16000;
  $dbh->{LongTruncOk} = 1;

  $iso_level = "SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED"; 
  &msg("  Setting isolation level: $iso_level"); 
  $dbh->do($iso_level) || &error("SET ISO LEVEL");
  &msg("connectDB() Done"); 

  $dbh->do("use tempdb") || &error("SET ISO LEVEL");
  ##$sql = " IF OBJECT_ID (N'tempdb.dbo.dateconvert_19', N'FN') IS NOT NULL DROP FUNCTION dbo.dateconvert_19; ";
  $sql = " IF OBJECT_ID (N'dbo.dateconvert_19', N'FN') IS NOT NULL DROP FUNCTION dbo.dateconvert_19; ";
  ##&msg("Running SQL=$sql");
  $sth = $dbh->do($sql) || &error("DROP_F1");

  $dbh->do("use tempdb") || &error("SET ISO LEVEL");
  #  -- 2005+ only?: WITH EXECUTE AS CALLER
  ##$sql = "CREATE FUNCTION dbo.dateconvert_19 (\@DATE datetime2)
  $sql = "CREATE FUNCTION dbo.dateconvert_19 (\@DATE datetime)
    RETURNS varchar(19)
    AS
    BEGIN
	 DECLARE \@RESULT varchar(19);
	 SET \@RESULT = CONVERT(varchar(10), \@DATE, 111) +
		  '-' + CONVERT(varchar(8),  \@DATE, 108);
	 RETURN(\@RESULT);
    END
  ";
  ##&msg("Running SQL=$sql");
  $sth = $dbh->do($sql) || &error("Create_F1");

  $connected_to_db = 1; 

  &connectVars;

  &msg("connectDB - Done"); 
}

# ----------------------------------------------------------------------
sub newVars
# ----------------------------------------------------------------------
{
  &msg("newVars - Starting"); 

  #---- Copy newly coded vars to the OLD NAME
  $ver      = $sql_version; 
  $vershort = $sql_vershort; 
  $PERLPATH = $new_PERLPATH; 
  $SQLPATH  = $sql_sqlpath; 
  $osver    = $sql_osver; 

  #---- 2013/06/08 New way using TieRegistry 
  if (! $osver   && $owmi_osver) { $osver   = $owmi_osver; }
  if (! $sqlpath && $rr_sqlpath) { $SQLPATH = $rr_sqlpath; }

  #---- Return values
  &dataReturn("OSVer:    $osver $osname");
  &dataReturn("PERLPATH: $PERLPATH");

  &msg("newVars - Done - \nver=$ver \nvershort=$vershort \nPERLPATH=$PERLPATH \nSQLPATH=$SQLPATH \nosver=$osver") unless $WWW; 
}

# ----------------------------------------------------------------------
sub checkVars
# ----------------------------------------------------------------------
{
  &msg("checkVars - Starting"); 

  my $var_errors = ""; 

  if ($SQLPATH eq "")  { $var_errors .= "SQLPATH is NULL,";  }
  if ($PERLPATH eq "") { $var_errors .= "PERLPATH is NULL,";  }
  if ($ver eq "")      { $var_errors .= "ver is NULL,";  }
  if ($vershort eq "") { $var_errors .= "vershort is NULL,";  }
  #if ($osver eq "")    { $var_errors .= "osver is NULL,";  }

  if ($var_errors) 
  {
    chop $var_errors; 
    &error("Null Required Variables: $var_errors"); 
  }

  &msg("checkVars - Done"); 
}

# ----------------------------------------------------------------------
sub connectVars
# ----------------------------------------------------------------------
{
  &msg("connectVars - Starting"); 

  $cv_debug = 0; 

  #---- Query SQL EDITION now, since it will be needed for other checks
  &msg("  Querying SQL for patchlevel and edition") if $cv_debug; 
  $dbh->do("use master")                || &dbError("Use_Master");
  $sth = $dbh->prepare("
    SELECT SERVERPROPERTY('productversion'), 
    SERVERPROPERTY ('productlevel'), 
    SERVERPROPERTY ('edition')
  ") || &dbError("Prepare_SQLVER_L3034");
  $sth->execute                         || &dbError("Execute_Select");
  $sql_version = $sql_patchlevel = $sql_edition = "?"; 
  while (@Data = $sth->fetchrow_array)
  {
    $sql_version      = $Data[0]; 
    $sql_patchlevel   = $Data[1]; 
    $sql_edition      = $Data[2]; 
  }
  $sth->finish() || &dbError("Could not execute FINISH");

  #---- Edit VERSHORT
  $sql_vershort = (split(/\./,$sql_version)) [0] . '.' .
                  (split(/\./,$sql_version)) [1]; 
  $sql_vershort =~ s/\.//;

  #---- VERSHORT could be in the format 800, 900 or 1000, etc.. edit that to be 80, 90, 100, 105
  if    ($sql_version =~ /^8.0/)      { $sql_vershort = 80; }
  elsif ($sql_version =~ /^9.0/)      { $sql_vershort = 90; }
  elsif ($sql_version =~ /^10.0/)     { $sql_vershort = 100; }
  elsif ($sql_version =~ /^10.5/)     { $sql_vershort = 105; }
  elsif ($sql_version =~ /^11.0/)     { $sql_vershort = 110; }
  else    { &error("DBAmon does not support for sql_version=$sql_version"); } 

  #---- REGREAD Call
  ##if ($use_mssqlregread && $owmi_version eq "") 
  if (! $use_wmiosver)
  {
    #---- OSVER
    &msg("MSSQL_REGREAD - osver"); 
    $sth = $dbh->prepare("
    exec master.dbo.xp_instance_regread
	N'HKEY_LOCAL_MACHINE',
	N'Software\\Microsoft\\Windows NT\\CurrentVersion',
	N'CurrentVersion'
    ") || &dbError("Prepare_SQLVER_L3691");
    $sth->execute || &dbError("Execute_Select");
    $sql_osver = "?"; 
    while (@Data = $sth->fetchrow_array)
    {
      $sql_osver    = $Data[1]; 
    }

    #---- sql_ospatch
    &msg("MSSQL_REGREAD - ospatch"); 
    $sth = $dbh->prepare("
    exec master.dbo.xp_instance_regread
	N'HKEY_LOCAL_MACHINE',
	N'Software\\Microsoft\\Windows NT\\CurrentVersion',
	N'CSDVersion'
    ") || &dbError("Prepare_SQLVER_L3691");
    $sth->execute || &dbError("Execute_Select");
    $sql_ospatch = "?"; 
    while (@Data = $sth->fetchrow_array)
    {
      $sql_ospatch    = $Data[1]; 
    }
    $sql_ospatch =~ s/Service Pack /SP/g;

    #---- Edit OSVER
    $sql_osver .= ",$sql_ospatch" if $sql_ospatch ne "?" and $sql_ospatch ne ""; 
  }

  if ($use_mssqlregread && $rr_sqlpath eq "") 
  {
    #---- SQLPATH
    &msg("MSSQL_REGREAD - sqlpath"); 
    $sth = $dbh->prepare("
    exec master.dbo.xp_instance_regread
	N'HKEY_LOCAL_MACHINE',
	N'Software\\Microsoft\\MSSQLServer\\Setup',
	N'SQLPath'
    ") || &dbError("Prepare_SQLVER_L3706");
    $sth->execute || &dbError("Execute_Select");
    $sql_sqlpath = "?"; 
    while (@Data = $sth->fetchrow_array)
    {
      $sql_sqlpath    = $Data[1]; 
    }
  }

  #---- SWKEY -- MAY NOT EXIST! -- We don't call this any more 2013/06
  #  $query_swkey = 0; 
  #  if ($query_swkey == 1) 
  #  {
  #    $sth = $dbh->prepare("
  #  exec master.dbo.xp_instance_regread
  #	N'HKEY_LOCAL_MACHINE',
  #	N'Software\\Microsoft\\Microsoft SQL Server\\$sql_vershort\\Registration',
  #	N'CD_KEY'
  #    ") || &dbError("Prepare_SQLVER_L3743");
  #    $sth->execute || &dbError("Execute_Select_L3811");
  #    $sql_swkey = "?"; 
  #    while (@Data = $sth->fetchrow_array)
  #    {
  #      $sql_swkey    = $Data[1]; 
  #    }
  #    print "MSSQLSWKEY: $mssql_regkey\n" if $sql_swkey && $sql_swkey ne "?";
  #  }

  if ($use_mssqlregread && $rr_svcacct eq "") 
  {
    #---- Service Account
    &msg("MSSQL_REGREAD - svcacct"); 
    $sth = $dbh->prepare("exec master.dbo.xp_instance_regread
	N'HKEY_LOCAL_MACHINE',
	N'SYSTEM\\CurrentControlSet\\Services\\MSSQLSERVER',
	N'ObjectName'
    ") || &dbError("Prepare_SAN_L3965");
    $sth->execute || &dbError("EXE_3966");
    $sql_service_account = "?"; 
    while (@Data = $sth->fetchrow_array)
    {
      $sql_service_account    = uc $Data[1]; 
    }
    &dataReturn("MSSQLServiceAccount:  $sql_service_account"); 
  }

  &msg("  Results - \nsql_version=($sql_version)\nsql_vershort=($sql_vershort)\nsql_patchlevel=($sql_patchlevel)\nsql_edition=($sql_edition)\nsql_osver=($sql_osver)\nsql_ospatch=($sql_ospatch)\nsql_sqlpath=($sql_sqlpath)\nsql_swkey=($sql_swkey)") if ! $WWW; 

  &msg("connectVars - Done"); 
}

# ----------------------------------------------------------------------
sub test
# ----------------------------------------------------------------------
{
  &msg("test()"); 

  &msg("Setting isolation level: SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED"); 
  $dbh->do("SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED") || &dbError("Use_Master_1");


  $dbname = "master";

	&msg("Running exec sp_spaceused for dbname='$dbname'"); 
	$sth = $dbh->prepare("exec sp_spaceused") || &dbError("Prepare");
	$dberror = 0;
	$sth->execute || &dbErrorOK("Connecting to db=$dbname");
	if ($dberror)
	{
	  ++$numdbsoffline;
	  print "DB:     $dbname_out CONNECTERROR \n";
	  next;
	}
	while (@Data = $sth->fetchrow_array)
	{
	  ##print "Data0=$Data[0]\n";   # Test
	  ##@words = split ' ', $Data[0];
	  ##$verb = $words[0];
	  ##if ($verb eq $dbname)
	  if (uc $Data[0] eq uc $dbname)
	  {
	    $db_spacealloc = $Data[1];
	    chop $db_spacealloc ; chop $db_spacealloc ;chop $db_spacealloc ;
	    $db_spacefree  = $Data[2];
	    chop $db_spacefree ; chop $db_spacefree ;chop $db_spacefree ;
	    $db_spaceused  = $db_spacealloc - $db_spacefree;
	  }
	}

  &msg("db_spaceused=$db_spaceused"); 
}

# ----------------------------------------------------------------------
sub reqFileRead 
# ----------------------------------------------------------------------
{
  #---- REQ Path, etc.
  $reqfile_path = "dbamon_oswin_req.pl";
  $reqfile_debug = 1; 

  &msg("reqFileRead - Starting - Reading reqfile_path=$reqfile_path dcf_debug=$dcf_debug"); 

  if (-e "$reqfile_path") 
  {
    &msg("Locating PERL and Syntax checking $reqfile_path"); 
    if (system("perl -v") == 0 && system("perl -wc $reqfile_path 2>&1") != 0)
    {
      &msg("PHONEHOME: Require file $reqfile_path Perl compile error - Not requiring"); 
    }
    elsif (-z $dcf_path)
    {
      &msg("PHONEHOME: Require file $reqfile_path Is zero bytes - Not requiring"); 
    }
    else
    {
      require "$reqfile_path"; 
    }
  }
  else
  {
    &msg("PHONEHOME: ReqFile $reqfile_path missing - REQFILEMISSING"); 
  }
  &msg("reqFileRead - Done"); 
}

# ----------------------------------------------------------------------
sub DCFRead 
# ----------------------------------------------------------------------
{
  #---- DCF (DBAmon control file) path
  $dcf_path = "dbamon_oswin_dcf_${in_instance}.txt";
  $dcf_debug = 1; 

  &msg("DCFRead - Starting - Reading dcf_path=$dcf_path dcf_debug=$dcf_debug"); 

  if (-e "$dcf_path") 
  {
    #---- Only require the file if it passes a perl syntax check
    ## if (system("perl -wc $dcf_path 2>&1") != 0)
    #---- 2012/07/30 BB - We only want to run this compile check IF we have the path to get to PERL (not always true - UKS2) 
    &msg("Locating PERL and Syntax checking $dcf_path"); 
    if (system("perl -v") == 0 && system("perl -wc $dcf_path 2>&1") != 0)
    {
      &msg("Require file $dcf_path Perl compile error - Not requiring"); 
      # system("echp ' ' | mailx -s 'DBAmon | $host/$os $dcf_path Compile Error' dbaadmin\@dba.it.mycompany.com") if $company eq "mycompany";
    }
    elsif (-z $dcf_path)
    {
      &msg("Require file $dcf_path Is zero bytes - Not requiring"); 
    }
    else
    {
      require "$dcf_path"; 
    }
  }
  else
  {
    &msg("Controlfile file $dcf_path missing - DCFFILEMISSING"); 
    # system("echo ' ' | mailx -s 'DBAmon | $host/$os - DCF $dcf_path Missing' dbaadmin\@dba.it.mycompany.com") if $company eq "mycompany";
  }

  #---- TEMP DEBUG: show variable values
  if ($dcf_debug) 
  {
    &msg("DEBUG - DCF Values read:"); 
    foreach $dcf_key (sort keys %dcf_data)
    {
      &msg("  dcf_data{$dcf_key}=$dcf_data{$dcf_key}"); 
    }
  }

  if ($dcf_data{"LastCycleFinishNum"} > 0)
  {
    $zmins_since = sprintf("%.2f", (time - $dcf_data{"LastCycleFinishNum"}) / 60);
    &msg("DCFRead - Done - Minutes since completion of last iteration: $zmins_since"); 
  }
}

#------------------------------------------------------------------------------
sub timeToText
#------------------------------------------------------------------------------
{
  my $zz_ts = $_[0];
  my ($sec,$min,$hour,$mday,$mon,$year) = localtime($zz_ts);
  $year += 1900;
  $mon += 1;

  # Pad with 0's
  my $yy = sprintf("%04d",$year);
  my $mm = sprintf("%02d",$mon);
  my $dd = sprintf("%02d",$mday);
  my $hh = sprintf("%02d",$hour);
  my $mi = sprintf("%02d",$min);
  my $ss = sprintf("%02d",$sec);

  my $date = "$yy/$mm/$dd-$hh:$mi:$ss";
  return $date;
}

# ----------------------------------------------------------------------
sub DCFWrite 
# ----------------------------------------------------------------------
{
  &msg("DCFWrite - Starting - Writing dcf_path=$dcf_path"); 
  $dcf_debug = 1; 

  #---- Cycle finish variable
  my $z = time; 
  $dcf_data{"LastCycleFinishNum"}  = $z; 
  $dcf_data{"LastCycleFinishChar"} = &timeToText($z); 

  #---- Write out values
  foreach $dcf_key (sort keys %dcf_data)
  {
    $dcf_out .= "\$dcf_data{$dcf_key} = '$dcf_data{$dcf_key}';\n";
    $dcf_count++; 
  }
  ##&msg("DEBUG - dcf_count=$dcf_count dcf_out=$dcf_out"); 

  open(DCF, ">$dcf_path") || &msg("Cannot open output $dcf_path - $!");
  print DCF $dcf_out; 
  close(DCF);

  system("type $dcf_path") if $dcf_debug;

  &msg("DCFWrite - Done - Wrote $dcf_count controlfile entries"); 
}

# ----------------------------------------------------------------------
sub init
# ----------------------------------------------------------------------
{
  &msg("init - Starting"); 

  $stime = time;

  $verb = $ARGV[0];
  $arg1 = $ARGV[1];
  $arg2 = $ARGV[2];
  $arg3 = $ARGV[3];
  $arg4 = $ARGV[4];

  $WWW = 1 if $arg1 eq "WWW";

  #---- Misc vars.
  ($pshort = $P) =~ s/.pl$//; 
  $plog =  "${pshort}_PgmLog.txt"; 
  open(PLOG,">$plog") || &error("Cannot open output plog=$plog - $!"); 
  &msg("  Starting PLOG"); 

  $userid = Win32::LoginName();
  $osname = Win32::GetOSName();
  $domain = Win32::DomainName();

  chop($host=`hostname`); 
  $host_uc = uc $host; 
  $fs = "<font size=2>"; 

  $system_root = $ENV{"SYSTEMROOT"};
  $root_fs = substr($system_root,0,2);

  #---- Read dbamon_oswin.cfg file
  &readCfg;

  #---- Read MSSQL11.MSSQL2 REG KEY for this instance 
  &regInit;

  #---- Old PERHPATH logic
  ##if    (-e "c:\\strawberry\\perl\\bin\\perl.exe")  { $new_PERLPATH = "c:\\strawberry\\perl\\bin\\perl.exe"; } 
  ##elsif (-e "c:\\perl\\bin\\perl.exe")              { $new_PERLPATH = "c:\\perl\\bin\\perl.exe"; } 
  ##elsif (-e "c:\\perl64\\bin\\perl.exe")            { $new_PERLPATH = "c:\\perl64\\bin\\perl.exe"; } 
  ##else                                              { $new_PERLPATH = "?"; } 
  #---- New PERHPATH logic
  $new_PERLPATH = $^X; 

  #---- Determine SHIFT
  $z1 = &get_date("Y/M/D-H:M:S");
  $now_hhmm = substr($z1,11,5); 
  ($zsec, $zmin, $zhr, $zday, $zmonth, $zyear, $now_weekday, $zdayofyr, $zjunk_yuk) = localtime(time);

  
  if ($now_hhmm ge "08:00" && $now_hhmm le "17:00" && $now_weekday != 0 && $now_weekday != 6)  
  { 
    $target_shift = "P";    #---- Prime shift
  } 
  elsif ($now_weekday == 0 || $now_weekday == 6)  
  { 
    $target_shift = "W";    #---- Weekend shift
  } 
  else                                               
  {
    $target_shift = "N";    #---- Night Weekday shift
  } 
  &msg("Shift calculations - now_hhmm=$now_hhmm now_weekday=$now_weekday target_shift=$target_shift"); 
  &dataReturn("TargetShift: $target_shift"); 
  
  #---- Display results
  &msg("  host=$host osname=$osname userid=$userid instance=($in_instance) domain=$domain pshort=$pshort");
  &msg("  Results - \nhost=($host)\nverb=($verb)\narg1=($arg1)\narg2=($arg2)\narg3=($arg3)\narg4=($arg4)\ndriver=($driver)\nnew_PERLPATH={$new_PERLPATH)\nrr_regkey=$rr_regkey\nrr_regpre=$rr_regpre") if ! $WWW;

  &msg("init - Done"); 
}

# --------------------------------------------------------------------
sub regInit
# --------------------------------------------------------------------
#  Find the MSSQL11.MSSQL2 REG KEY for this instance. This will be used
#  later.
# --------------------------------------------------------------------
{
  &msg("regInit - Starting"); 

  #---- This will be used later
  $rr_regkey = &regRead("HKEY_LOCAL_MACHINE/SOFTWARE/Microsoft/Microsoft SQL Server/Instance Names/SQL", "/$in_instance");
  $rr_regpre = "HKEY_LOCAL_MACHINE/SOFTWARE/Microsoft/Microsoft SQL Server" if $rr_regkey;

  if ($rr_regkey eq "")   #---- Try Wow6432 MODE - New Way
  {
    &msg("rr_regkey is Null - Trying Wow6432 tree"); 
    $rr_regkey = &regRead("HKEY_LOCAL_MACHINE/SOFTWARE/Microsoft/Microsoft SQL Server/Instance Names/SQL", "/$in_instance", "wow6432");
    if ($rr_regkey)
    {
      $rr_regpre = "HKEY_LOCAL_MACHINE/SOFTWARE/Microsoft/Microsoft SQL Server" if $rr_regkey;
      $use_rr_wow6432   = 1; 
      &msg("Reading REG in WOW6432 mode worked"); 
    }
  }

  #if ($rr_regkey eq "")   #---- Try Wow6432 - OLD
  #{
  #  &msg("rr_regkey is Null - Trying Wow6432 tree"); 
  #  $rr_regkey = &regRead("HKEY_LOCAL_MACHINE/SOFTWARE/Wow6432Node/Microsoft/Microsoft SQL Server/Instance Names/SQL", "/$in_instance");
  #  $rr_regpre = "HKEY_LOCAL_MACHINE/SOFTWARE/Wow6432Node/Microsoft/Microsoft SQL Server" if $rr_regkey;
  #}

  if ($rr_regkey eq "")   #---- Try SQL2000
  {
    &msg("rr_regkey is Null - Trying SQL2000 Format tree"); 
    $rr_regtest = &regRead("HKEY_LOCAL_MACHINE/SOFTWARE/Microsoft/Microsoft SQL Server/$in_instance/Setup", "/SQLPath");
    if ($rr_regtest ne "") 
    {
      $rr_regpre = "HKEY_LOCAL_MACHINE/SOFTWARE/Microsoft/Microsoft SQL Server";
      $rr_regkey = "$in_instance"; 
      &msg("Reading REG in SQL2000 mode worked"); 
    }
  }

  if ($rr_regkey eq "")   #---- Did we find it at all? 
  {
    &msg("Could not read rr_regkey"); 
    ##print "PHONEHOME:Could not find RR_REGKEY\n"; 
  }

  #----
  #---- Read some commonly used values that will be used later
  #----

  if ($rr_regpre) 
  {
    #---- Get SQLPATH
    $rr_sqlpath = &regRead("$rr_regpre/$rr_regkey/Setup", "/SQLPATH");

    #---- Get SQL Agent Errorlog Path
    $rr_sql_agent_log = &regRead("$rr_regpre/$rr_regkey/SQLServerAgent", "/ErrorLogFile");
  }

  #---- Get SQL Service Acct
  if ($default_instance) 
  {
    $rr_svcacct = &regRead("HKEY_LOCAL_MACHINE/System/CurrentControlSet/services/MSSQLSERVER", "/ObjectName");
  }
  else
  {
    $rr_svcacct = &regRead("HKEY_LOCAL_MACHINE/System/CurrentControlSet/services/MSSQL\$$in_instance", "/ObjectName");
  }

  #---- Some edits
  $rr_sqlpath =~ tr/\x00//d;

  #---- Do some data returns
  &dataReturn("MSSQLServiceAccount:  $rr_svcacct") if $rr_svcacct; 

  &msg("regInit - Done - \nrr_regpre=$rr_regpre\nrr_regkey=$rr_regkey\nrr_sqlpath=$rr_sqlpath\nrr_svcacct=$rr_svcacct\nrr_sql_agent_log=$rr_sql_agent_log") if ! $WWW; 
}

# --------------------------------------------------------------------
sub regRead
# --------------------------------------------------------------------
{
  my ($rr_folder, $rr_key, $rr_parm) = @_;

  my $rr_debug = 0;
  my $rr_return = "";

  &msg("regRead - Starting - rr_folder=$rr_folder rr_key=$rr_key") if $rr_debug;

  use Win32::TieRegistry( Delimiter=>"#", ArrayValues=>0 );
  my $pound              = $Registry->Delimiter("/");

  #---- From example at: http://stackoverflow.com/questions/1621053/perls-tieregistry-and-windows-server-2008-r2
  #---- $regKey = $Registry->Open("HKEY_LOCAL_MACHINE/SOFTWARE/Microsoft/MSSQLServer/Setup", {Access=> 0x20019|0x0100});
  #---- This syntax passes the HEX flag to access the 64 bit registry
  #----  $regKey = $Registry->Open($rr_folder, {Access=> 0x20019|0x0100});

  if ($rr_parm =~ /wow6432/i || $use_rr_wow6432)
  {
    $regKey = $Registry->Open($rr_folder, {Access=> 0x20019|0x0200});
  }
  elsif ($osname eq "Win2000")
  {
    $regKey = $Registry->Open($rr_folder, {Access=> 0x20019});
  }
  else
  {
    $regKey = $Registry->Open($rr_folder, {Access=> 0x20019|0x0100});
  }

  $rr_return = $regKey->{$rr_key};

  if ($rr_debug)
  {
    foreach $key (keys %$regKey) { print"DEBUG - $rr_folder / $key: " . $regKey->{"$key"} . "\n"; }
  }

  &msg("regRead - Done - returning (rr_return)=($rr_return)") if $rr_debug;
  return $rr_return;
}

# ----------------------------------------------------------------------
sub readCfg
# ----------------------------------------------------------------------
{
  &msg("readCfg - Starting"); 

  $file_cfg_local = "dbamon_oswin_local.cfg"; 
  $file_cfg = "dbamon_oswin.cfg"; 

  &msg("Checking existence of LOCAL CFG file $file_cfg_local"); 
  if (-e "$file_cfg_local" && -r "$file_cfg_local")
  {
    $file_cfg = $file_cfg_local; 
    &msg("LOCAL CFG File DOES exist - It will be used ($file_cfg)"); 
  }
  else
  {
    &msg("LOCAL CFG File does NOT exist - Default file will be used ($file_cfg)"); 
  }

  &msg("Reading - file_cfg=$file_cfg"); 

  $cnt_cfg = 0;
  $cfg_debug = 1; 

  if (-e "$file_cfg" && -r "$file_cfg")
  {
    open(CFG, "$file_cfg") || &error("Error: Count not open input file_cfg=$file_cfg - $!");
    while(<CFG>)
    {
      ##print;
      next if /^#/ || /^$/;
      $cnt_cfg++; 
      @words = split;
      if ($words[0] eq "CHKALERT" && $words[1] eq "E")
      {
        push (@cfg_chkalert_e, $words[2]); 
      }
      elsif ($words[0] eq "PERFCOUNTER" )
      {
	$zp1 = (split(/\|/,$_))[1];
	$zp2 = (split(/\|/,$_))[2];

        $cfg_perfcounter{$zp1} = $zp2; 
      }
      else
      {
        &msg("ERROR: Verb=$words[0] Noun1=$words[1] Is undefined"); 
      }
    }
    close(CFG); 
  }
  else
  {
    &msg("Config file $file_cfg does not exist or is not readable"); 
  }

  if ($cfg_debug)
  {
    if (! $WWW)
    {
      &msg("Resulting \@cfg_chkalert_e"); 
      foreach (@cfg_chkalert_e)
      {
	&msg("  $_");
      }

      &msg("Resulting \%cfg_perfcounter"); 
      foreach (sort keys %cfg_perfcounter)
      {
        $zp1 = $_; 
        $zp2 = $cfg_perfcounter{$_};
        print "key=($zp1),val=($zp2)\n";
      }
    }
  }

  #---- Test
  if (! $WWW)
  {
    &msg("From CFG file, these SQL Log errors will be excluded:"); 
    foreach (@cfg_chkalert_e)
    {
      &msg("  $_");
    }
  }

  &msg("readCfg - Done - Non-comment lines read: $cnt_cfg"); 
}

# ----------------------------------------------------------------------
sub getRegInfo
# ----------------------------------------------------------------------
{
  &msg("getRegInfo - Starting"); 

  #---- Old
  ##use Win32::Registry;
  #---- New
  eval "use Win32::Registry"; 
  &error("Could not USE Win32::Registry") if ($@);

  $debug_reg1 = 0;

  #---- Find $hostname
  %values="";
  $Register = "System\\CurrentControlset\\Services\\tcpip\\Parameters";
  $HKEY_LOCAL_MACHINE->Open($Register,$hkey)|| &msg("Cannot open registry 1 $Register - $!");
  $hkey->GetValues(\%values);
  foreach $value (keys(%values))
  {
    $RegType      = $values{$value}->[1];
    $RegValue     = $values{$value}->[2];
    $RegKey       = $values{$value}->[0];
    $host = $RegValue if $RegKey eq "Hostname";
  }
  $hkey->Close();
  &msg("DEBUG_REG1 - host=$host") if $debug_reg1;

  &msg("Reading registry for NETWORK DOMAIN"); 
  %values="";
  $Register = "System\\CurrentControlset\\Services\\tcpip\\Parameters";
  $HKEY_LOCAL_MACHINE->Open($Register,$hkey)|| &msg("Cannot open registry 2 $Register - $!");
  $hkey->GetValues(\%values);
  foreach $value (keys(%values))
  {
    $RegType      = $values{$value}->[1];
    $RegValue     = $values{$value}->[2];
    $RegKey       = $values{$value}->[0];
    $domain = $RegValue if $RegKey eq "Domain";
    $compwork=$domain;
  }
  $hkey->Close();

  #---- Save Company name
  @words = split(/\./,$compwork);
  $nexttolast = $#words - 1;
  $company = lc $words[$nexttolast];
  &msg("DEBUG_REG1 - company=$company") if $debug_reg1;

  #---- Get registry entries for Perl 
  &msg("Reading registry for PERL VERSION"); 
  %values="";
  $Register = "Software\\Perl";
  $HKEY_LOCAL_MACHINE->Open($Register,$hkey)|| &msg("Cannot open registry 3 $Register - $!");
  $hkey->GetValues(\%values);
  foreach $value (keys(%values))
  {
    $RegType      = $values{$value}->[1];
    $RegValue     = $values{$value}->[2];
    $RegKey       = $values{$value}->[0];
    $PERLPATH = $RegValue if $RegKey eq "";
  }
  $hkey->Close();
  &msg("DEBUG_REG1 - PERLPATH=$PERLPATH") if $debug_reg1;

  $path_length = length($PERLPATH);
  if (substr($PERLPATH,$path_length - 1,1) eq "\\")
  {
    chop($PERLPATH)
  }
  $PERLPATH = "$PERLPATH\\bin";
  if ($PERLPATH !~ /exe$/)
  {
    $PERLPATH .= "\\perl.exe";
  }

  #---- 
  #---- Find SQL values: SQLPATH, ver, vershort, mssql64bit
  #----
  $SQLPATH_2000 = ""; 
  $SQLPATH_2005 = ""; 
  $SQLPATH_WOW  = ""; 

  #---- Find SQL values (old way - 2000)

  &msg("Reading registry for SQL SQLPATH - Base"); 
  %values="";
  $Register = "Software\\Microsoft\\mssqlserver\\setup";
  $openkey = 1;
  $HKEY_LOCAL_MACHINE->Open($Register,$hkey) || ($openkey = 0);
  if ($openkey)
  {
    $hkey->GetValues(\%values);
    foreach $value (keys(%values))
    {
      $RegType      = $values{$value}->[1];
      $RegValue     = $values{$value}->[2];
      $RegKey       = $values{$value}->[0];
      ##print "$RegKey $RegValue \n";
      $SQLPATH = $RegValue if $RegKey eq "SQLPath";
    }
    $hkey->Close();
  }
  else 
  { 
    &msg("Could not find registry entries - Old style MSSQL entries (this is OK) - Register=$Register"); 
  }

  #---- Get MS SQL version OLD STYLE
  &msg("Reading registry for SQL SQLPATH - OLD STYLE"); 
  %values="";
  $Register = "Software\\Microsoft\\MSSQLServer\\MSSQLServer\\CurrentVersion";
  $HKEY_LOCAL_MACHINE->Open($Register,$hkey)|| &msg("Cannot open registry $Register - $!");
  if ($hkey)
  {
    $hkey->GetValues(\%values);
    foreach $value (keys(%values))
    {
      $RegType      = $values{$value}->[1];
      $RegValue     = $values{$value}->[2];
      $RegKey       = $values{$value}->[0];
      $VER_2000 = $RegValue if lc($RegKey) eq "csdversion";
      $VER_2000 = $RegValue if lc($RegKey) eq "currentversion" && $VER_2000 eq "";
    }
    $hkey->Close();
  }

  #---- 2011/09/29 BB: We have a SQL 2000 NAMED INSTANCE which does not use either of the above formats. Let's try this:
  if ($SQLPATH eq "")
  {
    &msg("Reading registry for SQL SQLPATH - OLD STYLE #2"); 
    %values="";
    $Register = "Software\\Microsoft\\Microsoft SQL Server\\$in_instance\\Setup";
    $openkey = 1;
    $HKEY_LOCAL_MACHINE->Open($Register,$hkey) || ($openkey = 0);
    if ($openkey)
    {
      $hkey->GetValues(\%values);
      foreach $value (keys(%values))
      {
	$RegKey    = $values{$value}->[0];
	$RegType   = $values{$value}->[1];
	$RegValue  = $values{$value}->[2];
	$SQLPATH   = $RegValue if $RegKey eq "SQLPath";
      }
      $hkey->Close();
    }
    &msg("Resulting SQLPATH=($SQLPATH)"); 

    #---- Get MS SQL version OLD STYLE
    &msg("Reading registry for SQL SQLPATH - OLD STYLE #3"); 
    %values="";
    $Register = "Software\\Microsoft\\Microsoft SQL Server\\$in_instance\\MSSQLServer\\CurrentVersion";
    $HKEY_LOCAL_MACHINE->Open($Register,$hkey)|| &msg("Cannot open registry $Register - $!");
    if ($hkey)
    {
      $hkey->GetValues(\%values);
      foreach $value (keys(%values))
      {
	$RegType      = $values{$value}->[1];
	$RegValue     = $values{$value}->[2];
	$RegKey       = $values{$value}->[0];
	$VER_2000 = $RegValue if lc($RegKey) eq "currentversion" && $VER_2000 eq "";
	$VER_2000 = $RegValue if lc($RegKey) eq "csdversion";
      }
      $hkey->Close();
    }
  }

  #---- Save this for later
  $SQLPATH_2000 = $SQLPATH; 
  $SQLPATH = ""; 

  #---- 2.01 - It may be that a server has both old and new format registry entries. We need to ALSO check for a named instance with this NAME.
  #----          if we don't find that, it's OK -- just use what we found above. 

  #---- 
  #---- Try NEW method (SQL2005, 2008, higher)
  #---- 

  #---- Find SQL instances 
  &msg("Reading registry for SQL SQLPATH - NEW STYLE #1"); 
  %values="";
  $Register = "Software\\Microsoft\\Microsoft SQL Server\\Instance Names\\SQL";
  $openkey = 1;
  $HKEY_LOCAL_MACHINE->Open($Register,$hkey) || ($openkey = 0);
  if ($openkey)
  {
    $hkey->GetValues(\%values);
    foreach $value (keys(%values))
    {
      $RegKey       = $values{$value}->[0];
      $RegType      = $values{$value}->[1];
      $RegValue     = $values{$value}->[2];
      ##print "key=$RegKey value=$RegValue \n";
      $sql_instances{$RegKey} = $RegValue if $RegKey;
    }
    $hkey->Close();
    &msg("MSSQL Instances Found:"); 
    foreach $this_instance (sort keys %sql_instances)
    {
      &msg("  SQL_Instance: $this_instance  SQL_Info: $sql_instances{$this_instance}");
    }

    #---- Now ensure that the $in_instance for this execution WAS found in the list of instances
    if (! $sql_instances{$in_instance} ) 
    { 
      &msg("The Instance that you specified ($in_instance) Does NOT EXIST on this server - It could be that this instance is OLD"); 
    }
    else
    {
      #---- If we are here, then this server DOES have a list of instances and this in_instance DOES exist
      $in_instance_key = $sql_instances{$in_instance}; 
      &msg("Using in_instance_key=$in_instance_key to find SQL version"); 

      #---- Get MS SQL version
      %values="";
      $Register = "Software\\Microsoft\\Microsoft SQL Server\\$in_instance_key\\MSSQLServer\\CurrentVersion";
      $HKEY_LOCAL_MACHINE->Open($Register,$hkey)|| die "Cannot open registry $Register - $!";
      $hkey->GetValues(\%values);
      foreach $value (keys(%values))
      {
	$RegType      = $values{$value}->[1];
	$RegValue     = $values{$value}->[2];
	$RegKey       = $values{$value}->[0];
	$VER_2005 = $RegValue if lc($RegKey) eq "csdversion";
	$VER_2005 = $RegValue if lc($RegKey) eq "currentversion" && $VER_2005 eq "";
      }
      $hkey->Close();

      #---- Get SQLPATH
      $SQLPATH = "";
      %values = "";
      $Register = "Software\\Microsoft\\Microsoft SQL Server\\$in_instance_key\\Setup";
      $HKEY_LOCAL_MACHINE->Open($Register,$hkey)|| die "Cannot open registry $Register - $!";
      $hkey->GetValues(\%values);
      foreach $value (keys(%values))
      {
	$RegType      = $values{$value}->[1];
	$RegValue     = $values{$value}->[2];
	$RegKey       = $values{$value}->[0];
	$SQLPATH = $RegValue if lc($RegKey) eq "sqlpath";
      }
      $hkey->Close();
    }
  }
  else 
  { 
    &msg("Could not read registry to get List of MSSQL instances - This could be an OLD server - Register=$Register"); 
  }
  #---- Save this for later
  $SQLPATH_2005 = $SQLPATH; 

  #----
  #---- Find SQL instances - WOW6432NODE method (rare)
  #----
  &msg("Reading registry for SQL SQLPATH - WOW6432NODE STYLE"); 
  %values="";
  $Register = "Software\\wow6432node\\Microsoft\\Microsoft SQL Server\\Instance Names\\SQL";
  $openkey = 1;
  $HKEY_LOCAL_MACHINE->Open($Register,$hkey) || ($openkey = 0);
  if ($openkey)
  {
    $hkey->GetValues(\%values);
    foreach $value (keys(%values))
    {
      $RegKey       = $values{$value}->[0];
      $RegType      = $values{$value}->[1];
      $RegValue     = $values{$value}->[2];
      ##print "key=$RegKey value=$RegValue \n";
      $sql_instances{$RegKey} = $RegValue if $RegKey;
    }
    $hkey->Close();
    &msg("MSSQL Instances Found (if any):"); 
    foreach $this_instance (sort keys %sql_instances)
    {
      &msg("  SQL_Instance: $this_instance  SQL_Info: $sql_instances{$this_instance}");
    }

    #---- Now ensure that the $in_instance for this execution WAS found in the list of instances
    if (! $sql_instances{$in_instance} ) 
    { 
      &msg("The Instance that you specified ($in_instance) Does NOT EXIST on this server in this registry mode (No problem)"); 
    }
    else
    {
      #---- If we are here, then this server DOES have a list of instances and this in_instance DOES exist
      $in_instance_key = $sql_instances{$in_instance}; 
      &msg("Using in_instance_key=$in_instance_key to find SQL version"); 

      #---- Get MS SQL version
      %values="";
      $Register = "Software\\wow6432node\\Microsoft\\Microsoft SQL Server\\$in_instance_key\\MSSQLServer\\CurrentVersion";
      $HKEY_LOCAL_MACHINE->Open($Register,$hkey)|| die "Cannot open registry $Register - $!";
      $hkey->GetValues(\%values);
      foreach $value (keys(%values))
      {
	$RegType      = $values{$value}->[1];
	$RegValue     = $values{$value}->[2];
	$RegKey       = $values{$value}->[0];
	$VER_WOW = $RegValue if lc($RegKey) eq "csdversion";
	$VER_WOW = $RegValue if lc($RegKey) eq "currentversion" && $VER_WOW eq "";
      }
      $hkey->Close();

      #---- Get SQLPATH_WOW
      $SQLPATH_WOW = "";
      %values = "";
      $Register = "Software\\wow6432node\\Microsoft\\Microsoft SQL Server\\$in_instance_key\\Setup";
      $HKEY_LOCAL_MACHINE->Open($Register,$hkey)|| die "Cannot open registry (wow6432node Mode) - Register=$Register - $!";
      $hkey->GetValues(\%values);
      foreach $value (keys(%values))
      {
	$RegType      = $values{$value}->[1];
	$RegValue     = $values{$value}->[2];
	$RegKey       = $values{$value}->[0];
	$SQLPATH_WOW  = $RegValue if lc($RegKey) eq "sqlpath";
      }
      $hkey->Close();
    }
  }
  else 
  { 
    &msg("Could not read registry to get List of MSSQL instances in this mode - Register=$Register - (No problem)"); 
  }

  #---- Decide which value to use for SQLPATH
  &msg("After REG samples - SQLPATH_2000=$SQLPATH_2000 SQLPATH_2005=$SQLPATH_2005 SQLPATH_WOW=$SQLPATH_WOW - Deciding which one to use..."); 
  if    ($SQLPATH_2005) 
  { 
    $SQLPATH = $SQLPATH_2005; 
    $ver     = $VER_2005; 
    &msg("Using SQLPATH_2005 SQLPATH value"); 
  }
  elsif ($SQLPATH_2000) 
  { 
    $SQLPATH = $SQLPATH_2000; 
    $ver     = $VER_2000; 
    &msg("Using SQLPATH_2000 SQLPATH value"); 
  }
  elsif ($SQLPATH_WOW) 
  { 
    $SQLPATH = $SQLPATH_WOW; 
    $ver     = $VER_WOW; 
    &msg("Using SQLPATH_WOW SQLPATH value"); 
  }
  else  { &error("Could NOT FIND SQLPATH from registry - Maybe you installed Perl 32-bit in a 64-bit OS?"); }

  #---- See if we can see if this is 64bit mode
  #---- I will try to make this work from the MASTER DB (that would be better)
  #$key = "HKEY_LOCAL_MACHINE\\software\\wow6432node\\microsoft\\microsoft sql server\\100\\tools\\setup";
  #foreach (`reg query \"$key\" `)
  #{
  #  #print;
  #  @words = split;
  #  if (/64-bit/) { $mssql64bit = 1; }
  #}
  #&msg( "DEBUG mssql64bit=$mssql64bit"); 
     
  if (length((split(/\./,$ver)) [2]) == 3)
  {
    $ver = (split(/\./,$ver)) [0] . '.' .
	   (split(/\./,$ver)) [1] . '.' .
	   "0" . (split(/\./,$ver)) [2];
    &msg("DEBUG = Padding right node of version with 0 - ver=$ver"); 
  }

  #---- If after this we don't have SQLPATH, look for the values in this file
  $sqlpathfile = "c:\\SQLPATH.txt";
  if ($SQLPATH eq "" && -e "$sqlpathfile")
  {
    &msg("SQLPATH Being set from file $sqlpathfile"); 
    chop($SQLPATH = `type $sqlpathfile`); 
    &msg("New SQLPATH=$SQLPATH"); 
  }

  $vershort = (split(/\./,$ver)) [0] . '.' .
	      (split(/\./,$ver)) [1]; 
  $vershort =~ s/\.//;

  &msg("DEBUG - vershort before: $vershort"); 
  #---- VERSHORT could be in the format 800, 900 or 1000, etc.. edit that to be 80, 90, 100, 105
  if    ($vershort == 800)  { $vershort = 80; }
  elsif ($vershort == 900)  { $vershort = 90; }
  elsif ($vershort == 1000) { $vershort = 100; }
  elsif ($vershort == 1050) { $vershort = 105; }
  elsif ($vershort == 1100) { $vershort = 110; }
  &msg("DEBUG - vershort after: $vershort"); 

  #---- Format VERSION
  #if    ($ver =~ /^8.0/)      { $ver_major = "2000"; }
  #elsif ($ver =~ /^9.0/)      { $ver_major = "2005"; }
  #elsif ($ver =~ /^10.0/)     { $ver_major = "2008"; }
  #elsif ($ver =~ /^10.5/)     { $ver_major = "2008R2"; }
  #else                        { $ver_major = ""; } 
  ##
  #if    ($ver eq "8.00.0384") { $ver_sp = "SP1"; }
  #elsif ($ver eq "8.00.0534") { $ver_sp = "SP2";  }
  #elsif ($ver =~ /9.00.2047/) { $ver_sp = "SP1";  }
  #elsif ($ver =~ /9.00.2047/) { $ver_sp = "SP1";  }
  #elsif ($ver =~ /10.0.2531/) { $ver_sp = "SP1";  }
  #else                        { $ver_sp = ""; } 
  ##
  #$ver = "$ver($ver_major($ver_sp))"; 

  &error("Could not find SQLPATH") if ! $SQLPATH;

  $SQLPATH =~ s/\x0//g if $SQLPATH;    #---- Remove any NULLS !!!
  &msg("SQLPATH=($SQLPATH) PERLPATH=$PERLPATH ver=$ver WWW=$WWW userid=$userid vershort=$vershort mssql64bit=$mssql64bit");
  &msg("getRegInfo - Done - company=$company"); 
}

#----------------------------------------------------------------------------------
sub WMIProcessor
#----------------------------------------------------------------------------------
{
  return if $WWW; 
  &msg("WMIProcessor - starting");

  use Win32::OLE('in');
 
  #use constant wbemFlagReturnImmediately => 0x10;
  #use constant wbemFlagForwardOnly => 0x20;
 
  my $computer = "localhost";
 
  my $objWMIService = Win32::OLE->GetObject("winmgmts:\\\\$computer\\root\\CIMV2"); 

  if (! $objWMIService)
  {
    &msg("objWMIService is NULL - Error=($!) - Returning");
    &setlocale_C;  #---- Reset locale in case OLE changed it to the local setting
    return;
  }

  #my $colItems = $objWMIService->ExecQuery("SELECT * FROM Win32_Processor", "WQL",
  #my $colItems = $objWMIService->ExecQuery("SELECT AddressWidth,Description, Name, MaxClockSpeed FROM Win32_Processor", "WQL",
  #my $colItems = $objWMIService->ExecQuery("SELECT AddressWidth,Description, Name, MaxClockSpeed, NumberOfCores, NumberOfLogicalProcessors FROM Win32_Processor", "WQL",
  my $colItems = $objWMIService->ExecQuery("SELECT AddressWidth,Description, Name, MaxClockSpeed FROM Win32_Processor", "WQL",
    wbemFlagReturnImmediately | wbemFlagForwardOnly);
 
  foreach my $objItem (in $colItems)
  {
    $pwmi_awidth      = $objItem->{AddressWidth};
    $pwmi_desc        = $objItem->{Description};
    $pwmi_maxclockmhz = $objItem->{MaxClockSpeed};
    $pwmi_name        = $objItem->{Name};
  }
 
  $pwmi_cpu_model = (split(/\s/,$pwmi_desc))[0];

  $pwmi_name =~ s/\s/_/g;
  $pwmi_desc =~ s/\s/_/g;

  &dataReturn("CPUModel:         $pwmi_cpu_model $pwmi_desc") if ! $usewin32registry; 
  &dataReturn("CPUClockSpeedMHz: $pwmi_maxclockmhz"); 
  &dataReturn("CPUArch:          $pwmi_awidth"); 

  if ($pwmi_awidth == 64) 
  {
    $os64bit = 1;
  }
  else
  {
    $os64bit = 0;
  }

  &msg("  Results - \npwmi_cpu_model=$pwmi_cpu_model \npwmi_awidth=$pwmi_awidth \npwmi_maxclockmhz=$pwmi_maxclockmhz \npwmi_desc=$pwmi_desc \npwmi_name=$pwmi_name
pwmi_num_cores=$pwmi_num_cores\npwmi_num_proc=$pwmi_num_proc");
  &msg("WMIProcessor - Done");
}

#----------------------------------------------------------------------------------
sub WMIComputerSystem
#----------------------------------------------------------------------------------
{
  return if $WWW; 
  &msg("WMIComputerSystem - starting");

  use Win32::OLE('in');
 
  #use constant wbemFlagReturnImmediately => 0x10;
  #use constant wbemFlagForwardOnly => 0x20;
 
  my $computer = "localhost";
 
  my $objWMIService = Win32::OLE->GetObject("winmgmts:\\\\$computer\\root\\CIMV2"); 

  if (! $objWMIService)
  {
    &msg("objWMIService is NULL - Error=($!) - Returning");
    &setlocale_C;  #---- Reset locale in case OLE changed it to the local setting
    return;
  }

  if ($owmi_osver_short < 6) 
  {
    $colname = "NumberOfProcessors"; 
  }
  else
  {
    $colname = "NumberOfLogicalProcessors"; 
  }

  my $colItems = $objWMIService->ExecQuery("SELECT $colname FROM Win32_ComputerSystem", "WQL",
    wbemFlagReturnImmediately | wbemFlagForwardOnly);

  $pwmi_num_proc = -1; 
 
  foreach my $objItem (in $colItems)
  {
    $pwmi_num_proc    = $objItem->{NumberOfProcessors}        if $objItem->{NumberOfProcessors} ne "";
    $pwmi_num_proc    = $objItem->{NumberOfLogicalProcessors} if $objItem->{NumberOfLogicalProcessors} ne "";
  }
 
  if ($pwmi_num_proc > 0) 
  {
    $numcpus = $pwmi_num_proc; 
    &dataReturn("NumCPUs: $numcpus");
  }

  &msg("WMIComputerSystem - Done - numcpus=$numcpus pwmi_num_proc=$pwmi_num_proc");
}

#----------------------------------------------------------------------------------
sub WMIOperatingSystem
#----------------------------------------------------------------------------------
{
  return if $WWW; 
  &msg("WMIOperatingSystem - Starting");

  #---- See: http://msdn.microsoft.com/en-us/library/windows/desktop/aa394239(v=vs.85).aspx 

  use Win32::OLE('in');
 
  my $computer = "localhost";
  my $objWMIService = Win32::OLE->GetObject("winmgmts:\\\\$computer\\root\\CIMV2"); 
  my $owmi_lastreboot = ""; 

  if (! $objWMIService)
  {
    &msg("objWMIService is NULL - Error=($!) - Returning");
    &setlocale_C;  #---- Reset locale in case OLE changed it to the local setting
    return;
  }

  my $colItems = $objWMIService->ExecQuery("SELECT lastbootuptime, systemdirectory, totalvisiblememorysize, freephysicalmemory FROM Win32_OperatingSystem", "WQL",
    wbemFlagReturnImmediately | wbemFlagForwardOnly);
 
  foreach my $objItem (in $colItems)
  {
    $owmi_lastreboot_raw = $objItem->{LastBootupTime};
    $owmi_systemdir      = $objItem->{SystemDirectory};
    $owmi_memtot         = $objItem->{TotalVisibleMemorySize};
    $owmi_memfree        = $objItem->{FreePhysicalMemory};
  }

  $owmi_memtot_mb  = int($owmi_memtot / 1024); 
  $owmi_memfree_mb = int($owmi_memfree / 1024); 

  &msg("DEBUG - owmi_lastreboot_raw=$owmi_lastreboot_raw owmi_systemdir=$owmi_systemdir owmi_memtot_mb=$owmi_memtot_mb owmi_memfree_mb=$owmi_memfree_mb");  

  $owmi_memused_mb = $owmi_memtot_mb - $owmi_memfree_mb;
  $owmi_mem_pct = sprintf("%.2f", (($owmi_memused_mb / $owmi_memtot_mb) * 100)) if $owmi_memtot_mb > 0; 
  
  &dataReturn("RAM:       $owmi_memtot_mb");
  &dataReturn("MEM_AVAIL: $owmi_memfree_mb");
  &dataReturn("MEM_USED:  $owmi_memused_mb");
  &dataReturn("MEM_PCT:   $owmi_mem_pct");

  #---- Memory:
  # dbamon_mssql.pl RAM:       8094
  # dbamon_mssql.pl MEM_AVAIL: 1941
  # dbamon_mssql.pl MEM_USED:  6153
  # dbamon_mssql.pl MEM_PCT:   76.02

  #---- Calc days since last reboot
  #----   Raw: 20120706150137.375199-360
  if ($owmi_lastreboot_raw =~ /^20/) 
  {
    $owmi_lastreboot = 
      substr($owmi_lastreboot_raw,0,4) . "/" . 
      substr($owmi_lastreboot_raw,4,2) . "/" .
      substr($owmi_lastreboot_raw,6,2) . "-" .
      substr($owmi_lastreboot_raw,8,2) . ":" .
      substr($owmi_lastreboot_raw,10,2) . ":" .
      substr($owmi_lastreboot_raw,12,2); 
    
    $owmi_lr_num = &timeToNum($owmi_lastreboot); 
    $owmi_lr_age_days = sprintf("%.2f", ((time() - $owmi_lr_num) / 86400)); 
  }
  else
  {
    $owmi_systemdir   = "?"; 
    $owmi_lastreboot  = "?"; 
    $owmi_lr_age_days = -1; 
  }

  &msg("DEBUG - owmi_lastreboot=$owmi_lastreboot owmi_lr_num=$owmi_lr_num owmi_lr_age_days=$owmi_lr_age_days");  
 
  print "$P OS_LastReboot:           $owmi_lastreboot\n"; 
  print "$P OS_LastRebootDaysSince:  $owmi_lr_age_days\n"; 
  print "$P OS_SystemDir:            $owmi_systemdir\n"; 

  &msg("WMIOperatingSystem - Done - owmi_lastreboot=$owmi_lastreboot owmi_systemdir=$owmi_systemdir");
}

#----------------------------------------------------------------------------------
sub WMIOSVer
#----------------------------------------------------------------------------------
{
  return if $WWW; 
  &msg("WMIOSVer - Starting");

  #---- See: http://msdn.microsoft.com/en-us/library/windows/desktop/aa394239(v=vs.85).aspx 

  use Win32::OLE('in');
 
  my $computer = "localhost";
  my $objWMIService = Win32::OLE->GetObject("winmgmts:\\\\$computer\\root\\CIMV2"); 

  if (! $objWMIService)
  {
    &msg("objWMIService is NULL - Error=($!) - Returning");
    &setlocale_C;  #---- Reset locale in case OLE changed it to the local setting
    return;
  }

  my $colItems = $objWMIService->ExecQuery("SELECT version, csdversion FROM Win32_OperatingSystem", "WQL",
    wbemFlagReturnImmediately | wbemFlagForwardOnly);
 
  foreach my $objItem (in $colItems)
  {
    $owmi_version    = $objItem->{Version};
    $owmi_csdversion = $objItem->{CSDVersion};
  }

  &msg("DEBUG - owmi_version=$owmi_version owmi_csdversion=$owmi_csdversion");  
 
  if ($owmi_csdversion ne "") { $owmi_csdversion =~ s/Service Pack /SP/g; } 

  #---- Form vars for use by other sprocs
  $owmi_osver = "$owmi_version,$owmi_csdversion"; 
  $owmi_osver_short = substr($owmi_osver,0,1); 

  #---- Return data to probe caller
  &dataReturn("OSVer:    $owmi_osver $osname");

  &msg("WMIOSVer - Done - owmi_osver_short=$owmi_osver_short");
}

#----------------------------------------------------------------------------------
sub WMISCSI
#----------------------------------------------------------------------------------
{
  return if $WWW; 
  &msg("WMISCSI - Starting");

  #---- See: http://msdn.microsoft.com/en-us/library/windows/desktop/aa394239(v=vs.85).aspx 

  use Win32::OLE('in');
 
  my $computer = "localhost";
  my $objWMIService = Win32::OLE->GetObject("winmgmts:\\\\$computer\\root\\CIMV2"); 

  if (! $objWMIService)
  {
    &msg("objWMIService is NULL - Error=($!) - Returning");
    &setlocale_C;  #---- Reset locale in case OLE changed it to the local setting
    return;
  }

  my $colItems = $objWMIService->ExecQuery("SELECT caption FROM Win32_scsicontroller", "WQL",
    wbemFlagReturnImmediately | wbemFlagForwardOnly);

  $owmi_scsi_cnt    = 0; 
  $owmi_scsi_pvscsi = 0; 
 
  foreach my $objItem (in $colItems)
  {
    $owmi_caption    = $objItem->{Caption};
    &msg("Found SCSI Controller: $owmi_caption"); 

    $owmi_scsi_cnt++;
    $owmi_scsi_pvscsi++ if $owmi_caption =~ /PVSCSI/; 
  }

  #---- Return data to probe caller
  &dataReturn("SYSPERF: SCSI_Cont_Tot     $owmi_scsi_cnt"); 
  &dataReturn("SYSPERF: SCSI_Cont_PVSCSI  $owmi_scsi_pvscsi"); 

  &msg("WMISCSI - Done - owmi_scsi_cnt=$owmi_scsi_cnt owmi_scsi_pvscsi=$owmi_scsi_pvscsi");
}

#----------------------------------------------------------------------------------
sub WMIService
#----------------------------------------------------------------------------------
{
  &msg("WMIService - Starting");

  #---- See: http://msdn.microsoft.com/en-us/library/windows/desktop/aa394239(v=vs.85).aspx 

  use Win32::OLE('in');
 
  my $computer = "localhost";
  my $objWMIService = Win32::OLE->GetObject("winmgmts:\\\\$computer\\root\\CIMV2"); 

  if (! $objWMIService)
  {
    &msg("objWMIService is NULL - Error=($!) - Returning");
    &setlocale_C;  #---- Reset locale in case OLE changed it to the local setting
    return;
  }

  my $colItems = $objWMIService->ExecQuery("SELECT name, started, startmode, caption FROM Win32_Service", "WQL",
    wbemFlagReturnImmediately | wbemFlagForwardOnly);

  my $svc_running = 0; 
  my $svc_notrunning = 0; 
  $svc_notrunning_autostart = 0; 
  $svc_notrunning_autostart_list = ""; 

  $services_running{"OSWin"}++; 
 
  foreach my $objItem (in $colItems)
  {
    $owmi_svc_name      = $objItem->{Name};
    $owmi_svc_started   = $objItem->{Started};
    $owmi_svc_startmode = $objItem->{StartMode};
    $owmi_svc_caption   = $objItem->{Caption};

    &msg("DEBUG - owmi_svc_name=$owmi_svc_name owmi_svc_started=$owmi_svc_started owmi_svc_startmode=$owmi_svc_startmode");
    
    $wmi_services{$owmi_svc_name} = $owmi_svc_started; 

    $svc_running++    if $owmi_svc_started == 1; 
    $svc_notrunning++ if $owmi_svc_started == 0; 

    #---- Only look for SQL services
    if ($owmi_svc_name =~ /sql/i && $owmi_svc_started == 0 && $owmi_svc_startmode eq "Auto") 
    {
      $svc_notrunning_autostart++; 
      $svc_notrunning_autostart_list .= "$owmi_svc_name,"; 
    }
    
    if ($owmi_svc_caption =~ /^SQL Server \(/ && $owmi_svc_started == 1)          {  $services_running{"MSSQL"}++; }
    if ($owmi_svc_caption =~ /^SQL Server Agent/ && $owmi_svc_started == 1)       {  $services_running{"MSSQL-Agent"}++; }
    if ($owmi_svc_caption =~ /^SQL Server Integration/ && $owmi_svc_started == 1) {  $services_running{"SSIS"}++; }
    if ($owmi_svc_caption =~ /^SQL Server Reporting/ && $owmi_svc_started == 1)   {  $services_running{"SSRS"}++;  }
    if ($owmi_svc_caption =~ /^SQL Server Anal/ && $owmi_svc_started == 1)        {  $services_running{"SSAS"}++; }
    if ($owmi_svc_caption =~ /^SQL Server Browser/ && $owmi_svc_started == 1)     {  $services_running{"MSSQL-Browser"}++; }
  }

  if ($svc_notrunning_autostart > 0)  
  {
    $svc_notrunning_autostart_list =~ s/,$//; 
    &dataReturn("SvcsNotRunningAS: $svc_notrunning_autostart $svc_notrunning_autostart_list");
  }

  #---- Create services running list
  $services_running_out = ""; 
  foreach $this_svc (sort keys %services_running) 
  {
    $services_running_out .= "$this_svc "; 
  }
  &dataReturn("SvcsString: $services_running_out"); 

  &msg("WMIService - Done - svc_running=$svc_running svc_notrunning=$svc_notrunning svc_notrunning_autostart=$svc_notrunning_autostart");
}

#----------------------------------------------------------------------------------
sub WMITopProcessesCPU
#----------------------------------------------------------------------------------
{
  return if $WWW; 

  &msg("WMITopProcessesCPU - Starting");

  #---- See: http://msdn.microsoft.com/en-us/library/windows/desktop/aa394239(v=vs.85).aspx 

  use Win32::OLE('in');
 
  my $computer = "localhost";
  my $objWMIService = Win32::OLE->GetObject("winmgmts:\\\\$computer\\root\\CIMV2"); 

  if (! $objWMIService)
  {
    &msg("objWMIService is NULL - Error=($!) - Returning");
    &setlocale_C;  #---- Reset locale in case OLE changed it to the local setting
    return;
  }

  #---- wmic path Win32_PerfFormattedData_PerfProc_Process get Name,PercentProcessorTime, workingset, privatebytes, workingsetpeak, pagefilebytes, elapsedtime
  my $colItems = $objWMIService->ExecQuery("SELECT Name, PercentProcessorTime, PrivateBytes, ElapsedTime, Description, IDProcess FROM Win32_PerfFormattedData_PerfProc_Process", "WQL",
    wbemFlagReturnImmediately | wbemFlagForwardOnly);

  my $tot_cpu = 0; 
  my $process_cnt = 0; 
 
  my $t_pct = 10; 
  my %pids = (); 

  $tot_processes_all  = 0; 
  $tot_processes_perl = 0; 
  $tot_processes_age_oldest_sec = 0; 

  foreach my $objItem (in $colItems)
  {

    $owmi_process_name    = $objItem->{Name};
    $owmi_cpu_pct         = $objItem->{PercentProcessorTime};
    $owmi_pid             = $objItem->{IDProcess};
    $owmi_elapsed_sec     = $objItem->{ElapsedTime}; 

    next if $owmi_process_name eq "Idle" || $owmi_process_name eq "_Total"; 

    $tot_processes_all++; 
    $tot_processes_age_oldest_sec = $owmi_elapsed_sec if $owmi_elapsed_sec > $tot_processes_age_oldest_sec;

    if ($owmi_process_name =~ /perl/i)
    {
      &msg("  Perl Process - owmi_process_name=$owmi_process_name owmi_cpu_pct=$owmi_cpu_pct owmi_elapsed_sec=$owmi_elapsed_sec"); 
      $tot_processes_perl++ if $owmi_process_name =~ /perl/i; 
    }
    

    next if $owmi_cpu_pct < $t_pct; 

    $process_cnt++; 

    &msg("  DEBUG - owmi_process_name=$owmi_process_name owmi_cpu_pct=$owmi_cpu_pct"); 

    $snap_cpdata = 
        "sn_process_name='$objItem->{Name}'<|>" . 
        "sn_cpu_pct='$objItem->{PercentProcessorTime}'<|>" . 
        "sn_private_bytes='$objItem->{PrivateBytes}'<|>" . 
        "sn_elapsed_seconds='$objItem->{ElapsedTime}'<|>" . 
        "sn_process_desc='$objItem->{Description}'<|>" .
        "sn_id_process='$objItem->{IDProcess}'<|>\n" ; 
    $pids{$owmi_pid} = $snap_cpdata;

    $tot_cpu += $owmi_cpu_pct; 
  }

  foreach $this_pid (sort keys %pids)
  {
    print "DDP:<|>$P<|>SNAP_CPU_PROCESS:<|>$pids{$this_pid}\n";
  }

  &dataReturn("SYSPERF: Processes_Count  $tot_processes_all"); 
  &dataReturn("SYSPERF: Processes_Perl   $tot_processes_perl"); 

  $server_upfor_days = sprintf("%.2f", $tot_processes_age_oldest_sec / 86400 ); 
  &dataReturn("SYSPERF: Server_Upfor_Days  $server_upfor_days"); 

  &msg("WMITopProcessesCPU - Done - tot_cpu=$tot_cpu process_cnt=$process_cnt tot_processes_all=$tot_processes_all tot_processes_perl=$tot_processes_perl tot_processes_age_oldest_sec=$tot_processes_age_oldest_sec");
}

#----------------------------------------------------------------------------------
sub WMINetPerf
#----------------------------------------------------------------------------------
{
  return if $WWW; 

  &msg("WMINetPerf - Starting");

  #---- See: http://msdn.microsoft.com/en-us/library/windows/desktop/aa394239(v=vs.85).aspx 

  use Win32::OLE('in');
 
  my $computer = "localhost";
  my $objWMIService = Win32::OLE->GetObject("winmgmts:\\\\$computer\\root\\CIMV2"); 

  if (! $objWMIService)
  {
    &msg("objWMIService is NULL - Error=($!) - Returning");
    &setlocale_C;  #---- Reset locale in case OLE changed it to the local setting
    return;
  }

  my $colItems = $objWMIService->ExecQuery("SELECT name, currentbandwidth, bytesreceivedpersec, bytessentpersec, packetsreceivederrors, packetsoutbounderrors, OutputQueueLength FROM Win32_PerfFormattedData_Tcpip_NetworkInterface", "WQL", wbemFlagReturnImmediately | wbemFlagForwardOnly);
 
  $wminet_rows = 0; 

  foreach $objItem (in $colItems)
  {  
    $wminet_rows++; 

    $wminp_name = $objItem->{name};
    &msg("  NetPerf Row - wminp_name=$wminp_name"); 

    #BytesReceivedPersec  BytesSentPersec  CurrentBandwidth   OutputQueueLength  PacketsOutboundErrors  PacketsReceivedErrors
    $wminp_ctrs{BytesReceivedPersec}   += $objItem->{BytesReceivedPersec};
    $wminp_ctrs{BytesSentPersec}       += $objItem->{BytesSentPersec};
    $wminp_ctrs{CurrentBandwidth}      += $objItem->{CurrentBandwidth};
    $wminp_ctrs{OutputQueueLength}     += $objItem->{OutputQueueLength};
    $wminp_ctrs{PacketsOutboundErrors} += $objItem->{PacketsOutboundErrors};
    $wminp_ctrs{PacketsReceivedErrors} += $objItem->{PacketsReceivedErrors};

    $wminp_ctrs{BytesTotalPersec}      =  $wminp_ctrs{BytesReceivedPersec} + $wminp_ctrs{BytesSentPersec}; 
  }

  &msg("Totals:"); 

  foreach $this_ctr (sort keys %wminp_ctrs)
  {
    $this_val = $wminp_ctrs{"$this_ctr"}; 
    &msg("  this_ctr=$this_ctr this_val=$this_val"); 
    &dataReturn("SYSPERF: NET_$this_ctr $this_val"); 
  }

  &msg("WMINetPerf - Done - wminet_rows=$wminet_rows");
}

# ----------------------------------------------------------------------
sub driveCheck
# ----------------------------------------------------------------------
{
  &msg("driveCheckWMI - Loading DRIVE data into hash"); 

  use Win32::OLE('in'); 

  #use constant wbemFlagReturnImmediately => 0x10; 
  #use constant wbemFlagForwardOnly => 0x20; 

  $computer = ".";
  $objWMIService = Win32::OLE->GetObject
      ("winmgmts:\\\\$computer\\root\\CIMV2");  #--- or die "WMI connection failed.\n";
 
  if (! $objWMIService)
  {
    &msg("objWMIService is NULL - Error=($!) - Running driveCheckCMD instead");
    &setlocale_C;  #---- Reset locale in case OLE changed it to the local setting
    &driveCheckCMD;
    return;
  }

  $colItems = $objWMIService->ExecQuery 
      ("SELECT deviceid, size, freespace, status, statusinfo, volumename FROM Win32_LogicalDisk where drivetype = 3","WQL",wbemFlagReturnImmediately | wbemFlagForwardOnly); 
   
  $drive_count = 0;
  $tot_mb_size = 0;
  $tot_mb_free = 0;
  undef %disks;

  &setlocale_C;  #---- Reset locale in case OLE changed it to the local setting

  foreach my $objItem (in $colItems) 
  { 
    #print "Device ID: $objItem->{DeviceID}\n"; 
    #print "Free Space: $objItem->{FreeSpace}\n"; 
    #print "Size: $objItem->{Size}\n"; 

    $drive = $objItem->{DeviceID}; 
    $drive =~ s/\://;

    #---- 1.01 2015/09/13 - Skip paging volumes
    $vname = $objItem->{VolumeName}; 
    &msg("DEBUG = drive=$drive vname=$vname"); 
    if ($vname =~ /pag/i) 
    {
      &msg("Drive=$drive is a PAG* volume - skipping"); 
      next; 
    }

    #---- Now that we know that we do want this volume, add it to the hash
    $disks{"$drive"} = 1;

    $disk_size_mb{"$drive"} = sprintf("%.2f", ($objItem->{Size}      / (1024*1024))  ); 
    $disk_free_mb{"$drive"} = sprintf("%.2f", ($objItem->{FreeSpace} / (1024*1024))  ); 
    $disk_used_mb{"$drive"} = sprintf("%.2f", ($disk_size_mb{"$drive"} - $disk_free_mb{"$drive"}));

    $disk_size_gb{"$drive"} = sprintf("%.2f", ($objItem->{Size}      / (1024*1024*1024))  ); 
    $disk_free_gb{"$drive"} = sprintf("%.2f", ($objItem->{FreeSpace} / (1024*1024*1024))  ); 
    $disk_used_gb{"$drive"} = sprintf("%.2f", ($disk_size_gb{"$drive"} - $disk_free_gb{"$drive"}));

    $disk_pct_full{"$drive"} = sprintf("%.2f", 100 * ($disk_used_gb{"$drive"} / $disk_size_gb{"$drive"})) if  $disk_size_gb{"$drive"} > 0;
    $drive_count++; 

    $tot_mb_size += $disk_size_mb{"$drive"}; 
    $tot_mb_free += $disk_free_mb{"$drive"}; 
  } 

  #---- See if we have ANY results? 
  if ($tot_mb_size == 0) 
  {
    &msg("Even after running WMI, tot_mb_size=$tot_mb_size - We will try driveCheckCMD instead");
    &driveCheckCMD;
    return;
  }

  $tot_gb_list = ""; 
  &msg("  Resulting Disk info:");
  foreach $zdrive (sort keys %disks)
  {
    &msg("    Disk: $zdrive - Size=$disk_size_gb{$zdrive} Free=$disk_free_gb{$zdrive} Used=$disk_used_gb{$zdrive}  PctF=$disk_pct_full{$zdrive}"); 
    $tot_gb_list .= $zdrive; 
    push(@db_drivelist, $zdrive);
  }

  #---- Return Total disk size
  my $tot_gb_size = int($tot_mb_size / 1024); 
  my $tot_gb_used = $tot_gb_size - ($tot_mb_free / 1024); 
  my $tot_gb_pct  = sprintf "%.2f", ($tot_gb_used / $tot_gb_size) * 100;
  &dataReturn("DISK_Tot_Size_GB: $tot_gb_size");  
  &dataReturn("DISK_Tot_Pct:     $tot_gb_pct");  
  &dataReturn("DISK_Tot_List:    $tot_gb_list");  

  &msg("driveCheckWMI - Done - tot_gb_size=$tot_gb_size tot_mb_size=$tot_mb_size tot_mb_free=$tot_mb_free"); 
}

# ----------------------------------------------------------------------
sub countCPUs
# ----------------------------------------------------------------------
{
  &msg("countCPUs - Starting - srvinfo_cmd=$srvinfo_cmd"); 

  if ($ENV{"NUMBER_OF_PROCESSORS"})
  {
    &msg("  Setting numcpus from ENV"); 
    $numcpus = $ENV{"NUMBER_OF_PROCESSORS"};
    #print "PHONEHOME:_NUMCPUs_from_ENV\n"; 
  }
  else
  {
    &msg("  Setting numcpus from $srvinfo_cmd"); 
    open(SRVINFO, "$srvinfo_cmd 2>&1 |") || &error("Error: Running $srvinfo_cmd");
    while(<SRVINFO>)
    {
      ##print;
      next unless /^CPU/;
      $numcpus++; 
    }
    &msg("  Resulting numcpus=$numcpus"); 
    #print "PHONEHOME:_NUMCPUs_from_SRVINFO\n"; 
  }

  print "PHONEHOME:_NUMCPUs_is_NULL\n" if ! $numcpus;

  &dataReturn("NumCPUs: $numcpus");

  &msg("countCPUs - Done - numcpus=$numcpus"); 
}

# ----------------------------------------------------------------------
sub driveCheckCMD
# ----------------------------------------------------------------------
{
  &msg("driveCheckCMD - Loading DRIVE data into hash - running srvinfo_cmd=$srvinfo_cmd"); 

  #print "PHONEHOME:driveCheckCMD\n"; 

  open(SRVINFO, "$srvinfo_cmd 2>&1 |") || &error("Error: Running $srvinfo_cmd");
  $drive_count = 0;
  $tot_mb_size = 0;
  $tot_mb_free = 0;
  undef %disks;
  while(<SRVINFO>)
  {
    ##print;
    @words = split;

    if (length($words[0]) == 2 && ($words[1] eq "FAT" || $words[1] eq "NTFS"))
    {
      $drive = substr($words[0],0,1);
      $disks{"$drive"} = $_;
      $disk_size_gb{"$drive"} = sprintf("%.2f",$words[2] / 1024);
      $disk_free_gb{"$drive"} = sprintf("%.2f",$words[3] / 1024);
      $disk_used_gb{"$drive"} = sprintf("%.2f", ($disk_size_gb{"$drive"} - $disk_free_gb{"$drive"}));
      $disk_size_mb{"$drive"} = sprintf("%.2f",$words[2] );
      $disk_free_mb{"$drive"} = sprintf("%.2f",$words[3] );
      $disk_used_mb{"$drive"} = sprintf("%.2f", ($disk_size_mb{"$drive"} - $disk_free_mb{"$drive"}));
      $disk_pct_full{"$drive"} = sprintf("%.2f", 100 * ($disk_used_gb{"$drive"} / $disk_size_gb{"$drive"})) if  $disk_size_gb{"$drive"} > 0;
      $drive_count++; 
      $tot_mb_size += $disk_size_mb{"$drive"}; 
      $tot_mb_free += $disk_free_mb{"$drive"}; 
    }
  }

  &msg("  Resulting Disk info:");
  foreach $zdrive (sort keys %disks)
  {
    &msg("    Disk: $zdrive - Size=$disk_size_gb{$zdrive} Free=$disk_free_gb{$zdrive} Used=$disk_used_gb{$zdrive}  PctF=$disk_pct_full{$zdrive}"); 
  }

  &msg("driveCheckCMD - Done - tot_mb_size=$tot_mb_size tot_mb_free=$tot_mb_free"); 
}


# ----------------------------------------------------------------------
sub setlocale_C
# ----------------------------------------------------------------------
{
  &msg("setlocale_C - Starting - Resetting LC_NUMERIC to C (just in case it changed)");
  use POSIX;
  setlocale(LC_NUMERIC, "C");
  &msg("setlocale_C - Done");
}

# ----------------------------------------------------------------------
sub dataReturn
# ----------------------------------------------------------------------
{
  my $in_data = $_[0]; 
  return if $WWW; 
  print "$P ${in_data}\n"; 
  return; 
}

#--------------------------------------------------------------------------
#
# Main
#
#--------------------------------------------------------------------------

&msg("Starting - \$0=$0 usewin32registry=$usewin32registry");

#---- Initial housekeeping
&init;
&DCFRead unless $WWW;
&reqFileRead unless $WWW;

#---- Find SQL instance, version, system stuff 
#&msg("DEBUG - usewin32registry=$usewin32registry"); 
#exit; 
#if ($usewin32registry)    #---- NEW LOGIC
#{
#  &getRegInfo;   #---- This is the OLD WAY
#}
#else
#{
#  &connectDB;     #---- This calls connectVars which reads variable values from the engine
#  &newVars;
#  #---- Make sure that all required vars are populated
#  &checkVars;
#}

#---- Check to see if server is up
if ($verb eq "up") 
{
  $subrc += &up();
}

#---- Test - Old Perl ProcessesBackup history
elsif ($verb eq "test2") 
{
  $subrc += &SysPerf; 
  $subrc += &SysPerfBulk; 
  $subrc += &SysPerfCompare; 
  $subrc += &SysPerfSpecial; 
}
else
{
  &error("Unknown verb=$verb passed"); 
}

#---- Terminate
$etime = time;
$elapsed = $etime - $stime;

&msg("Done - elapsed: $elapsed - subrc: $subrc");
&msg("END_OF_PROBE");

#---- Cleanup
close(PLOG); 
&DCFWrite unless $WWW;

#---- Terminate
exit 0;
